/****************************************************************************
** Meta object code from reading C++ file 'abstract3dcontroller_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/engine/abstract3dcontroller_p.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtDataVisualization/qbar3dseries.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstract3dcontroller_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20Abstract3DControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto Abstract3DController::qt_create_metaobjectdata<qt_meta_tag_ZN20Abstract3DControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Abstract3DController",
        "shadowQualityChanged",
        "",
        "QAbstract3DGraph::ShadowQuality",
        "quality",
        "activeInputHandlerChanged",
        "QAbstract3DInputHandler*",
        "inputHandler",
        "activeThemeChanged",
        "Q3DTheme*",
        "activeTheme",
        "selectionModeChanged",
        "QAbstract3DGraph::SelectionFlags",
        "mode",
        "needRender",
        "axisXChanged",
        "QAbstract3DAxis*",
        "axis",
        "axisYChanged",
        "axisZChanged",
        "elementSelected",
        "QAbstract3DGraph::ElementType",
        "type",
        "measureFpsChanged",
        "enabled",
        "currentFpsChanged",
        "fps",
        "orthoProjectionChanged",
        "aspectRatioChanged",
        "ratio",
        "horizontalAspectRatioChanged",
        "optimizationHintsChanged",
        "QAbstract3DGraph::OptimizationHints",
        "hints",
        "polarChanged",
        "radialLabelOffsetChanged",
        "offset",
        "reflectionChanged",
        "reflectivityChanged",
        "reflectivity",
        "localeChanged",
        "locale",
        "queriedGraphPositionChanged",
        "QVector3D",
        "data",
        "marginChanged",
        "margin",
        "destroyRenderer",
        "handleAxisTitleChanged",
        "title",
        "handleAxisLabelsChanged",
        "handleAxisRangeChanged",
        "min",
        "max",
        "handleAxisSegmentCountChanged",
        "count",
        "handleAxisSubSegmentCountChanged",
        "handleAxisAutoAdjustRangeChanged",
        "autoAdjust",
        "handleAxisLabelFormatChanged",
        "format",
        "handleAxisReversedChanged",
        "enable",
        "handleAxisFormatterDirty",
        "handleAxisLabelAutoRotationChanged",
        "angle",
        "handleAxisTitleVisibilityChanged",
        "visible",
        "handleAxisTitleFixedChanged",
        "fixed",
        "handleInputViewChanged",
        "QAbstract3DInputHandler::InputView",
        "view",
        "handleInputPositionChanged",
        "QPoint",
        "position",
        "handleSeriesVisibilityChanged",
        "handleThemeColorStyleChanged",
        "Q3DTheme::ColorStyle",
        "style",
        "handleThemeBaseColorsChanged",
        "QList<QColor>",
        "color",
        "handleThemeBaseGradientsChanged",
        "QList<QLinearGradient>",
        "gradient",
        "handleThemeSingleHighlightColorChanged",
        "QColor",
        "handleThemeSingleHighlightGradientChanged",
        "QLinearGradient",
        "handleThemeMultiHighlightColorChanged",
        "handleThemeMultiHighlightGradientChanged",
        "handleThemeTypeChanged",
        "Q3DTheme::Theme",
        "theme",
        "handleRequestShadowQuality",
        "updateCustomItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'shadowQualityChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::ShadowQuality)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'activeInputHandlerChanged'
        QtMocHelpers::SignalData<void(QAbstract3DInputHandler *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'activeThemeChanged'
        QtMocHelpers::SignalData<void(Q3DTheme *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'selectionModeChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::SelectionFlags)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'needRender'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'axisXChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'axisYChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'axisZChanged'
        QtMocHelpers::SignalData<void(QAbstract3DAxis *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'elementSelected'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::ElementType)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'measureFpsChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'currentFpsChanged'
        QtMocHelpers::SignalData<void(qreal)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 26 },
        }}),
        // Signal 'orthoProjectionChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'aspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 29 },
        }}),
        // Signal 'horizontalAspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 29 },
        }}),
        // Signal 'optimizationHintsChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::OptimizationHints)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Signal 'polarChanged'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'radialLabelOffsetChanged'
        QtMocHelpers::SignalData<void(float)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 36 },
        }}),
        // Signal 'reflectionChanged'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Signal 'reflectivityChanged'
        QtMocHelpers::SignalData<void(qreal)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 39 },
        }}),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void(const QLocale &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QLocale, 41 },
        }}),
        // Signal 'queriedGraphPositionChanged'
        QtMocHelpers::SignalData<void(const QVector3D &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Signal 'marginChanged'
        QtMocHelpers::SignalData<void(qreal)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 46 },
        }}),
        // Slot 'destroyRenderer'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAxisTitleChanged'
        QtMocHelpers::SlotData<void(const QString &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'handleAxisLabelsChanged'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAxisRangeChanged'
        QtMocHelpers::SlotData<void(float, float)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 52 }, { QMetaType::Float, 53 },
        }}),
        // Slot 'handleAxisSegmentCountChanged'
        QtMocHelpers::SlotData<void(int)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 55 },
        }}),
        // Slot 'handleAxisSubSegmentCountChanged'
        QtMocHelpers::SlotData<void(int)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 55 },
        }}),
        // Slot 'handleAxisAutoAdjustRangeChanged'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'handleAxisLabelFormatChanged'
        QtMocHelpers::SlotData<void(const QString &)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 },
        }}),
        // Slot 'handleAxisReversedChanged'
        QtMocHelpers::SlotData<void(bool)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 62 },
        }}),
        // Slot 'handleAxisFormatterDirty'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleAxisLabelAutoRotationChanged'
        QtMocHelpers::SlotData<void(float)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 65 },
        }}),
        // Slot 'handleAxisTitleVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 67 },
        }}),
        // Slot 'handleAxisTitleFixedChanged'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 69 },
        }}),
        // Slot 'handleInputViewChanged'
        QtMocHelpers::SlotData<void(QAbstract3DInputHandler::InputView)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Slot 'handleInputPositionChanged'
        QtMocHelpers::SlotData<void(const QPoint &)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'handleSeriesVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 67 },
        }}),
        // Slot 'handleThemeColorStyleChanged'
        QtMocHelpers::SlotData<void(Q3DTheme::ColorStyle)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 78, 79 },
        }}),
        // Slot 'handleThemeBaseColorsChanged'
        QtMocHelpers::SlotData<void(const QList<QColor> &)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 81, 82 },
        }}),
        // Slot 'handleThemeBaseGradientsChanged'
        QtMocHelpers::SlotData<void(const QList<QLinearGradient> &)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 84, 85 },
        }}),
        // Slot 'handleThemeSingleHighlightColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 87, 82 },
        }}),
        // Slot 'handleThemeSingleHighlightGradientChanged'
        QtMocHelpers::SlotData<void(const QLinearGradient &)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 89, 85 },
        }}),
        // Slot 'handleThemeMultiHighlightColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(90, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 87, 82 },
        }}),
        // Slot 'handleThemeMultiHighlightGradientChanged'
        QtMocHelpers::SlotData<void(const QLinearGradient &)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 89, 85 },
        }}),
        // Slot 'handleThemeTypeChanged'
        QtMocHelpers::SlotData<void(Q3DTheme::Theme)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 93, 94 },
        }}),
        // Slot 'handleRequestShadowQuality'
        QtMocHelpers::SlotData<void(QAbstract3DGraph::ShadowQuality)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'updateCustomItem'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Abstract3DController, qt_meta_tag_ZN20Abstract3DControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Abstract3DController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20Abstract3DControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20Abstract3DControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20Abstract3DControllerE_t>.metaTypes,
    nullptr
} };

void Abstract3DController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Abstract3DController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->shadowQualityChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::ShadowQuality>>(_a[1]))); break;
        case 1: _t->activeInputHandlerChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DInputHandler*>>(_a[1]))); break;
        case 2: _t->activeThemeChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme*>>(_a[1]))); break;
        case 3: _t->selectionModeChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::SelectionFlags>>(_a[1]))); break;
        case 4: _t->needRender(); break;
        case 5: _t->axisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 6: _t->axisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 7: _t->axisZChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 8: _t->elementSelected((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::ElementType>>(_a[1]))); break;
        case 9: _t->measureFpsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->currentFpsChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 11: _t->orthoProjectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->aspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 13: _t->horizontalAspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 14: _t->optimizationHintsChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::OptimizationHints>>(_a[1]))); break;
        case 15: _t->polarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->radialLabelOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 17: _t->reflectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->reflectivityChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 19: _t->localeChanged((*reinterpret_cast<std::add_pointer_t<QLocale>>(_a[1]))); break;
        case 20: _t->queriedGraphPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 21: _t->marginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 22: _t->destroyRenderer(); break;
        case 23: _t->handleAxisTitleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->handleAxisLabelsChanged(); break;
        case 25: _t->handleAxisRangeChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 26: _t->handleAxisSegmentCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->handleAxisSubSegmentCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->handleAxisAutoAdjustRangeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->handleAxisLabelFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->handleAxisReversedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->handleAxisFormatterDirty(); break;
        case 32: _t->handleAxisLabelAutoRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 33: _t->handleAxisTitleVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->handleAxisTitleFixedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->handleInputViewChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DInputHandler::InputView>>(_a[1]))); break;
        case 36: _t->handleInputPositionChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 37: _t->handleSeriesVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->handleThemeColorStyleChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme::ColorStyle>>(_a[1]))); break;
        case 39: _t->handleThemeBaseColorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 40: _t->handleThemeBaseGradientsChanged((*reinterpret_cast<std::add_pointer_t<QList<QLinearGradient>>>(_a[1]))); break;
        case 41: _t->handleThemeSingleHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 42: _t->handleThemeSingleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 43: _t->handleThemeMultiHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 44: _t->handleThemeMultiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 45: _t->handleThemeTypeChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme::Theme>>(_a[1]))); break;
        case 46: _t->handleRequestShadowQuality((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::ShadowQuality>>(_a[1]))); break;
        case 47: _t->updateCustomItem(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DInputHandler* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Q3DTheme* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DGraph::ShadowQuality )>(_a, &Abstract3DController::shadowQualityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DInputHandler * )>(_a, &Abstract3DController::activeInputHandlerChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(Q3DTheme * )>(_a, &Abstract3DController::activeThemeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DGraph::SelectionFlags )>(_a, &Abstract3DController::selectionModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)()>(_a, &Abstract3DController::needRender, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DAxis * )>(_a, &Abstract3DController::axisXChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DAxis * )>(_a, &Abstract3DController::axisYChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DAxis * )>(_a, &Abstract3DController::axisZChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DGraph::ElementType )>(_a, &Abstract3DController::elementSelected, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(bool )>(_a, &Abstract3DController::measureFpsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(qreal )>(_a, &Abstract3DController::currentFpsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(bool )>(_a, &Abstract3DController::orthoProjectionChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(qreal )>(_a, &Abstract3DController::aspectRatioChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(qreal )>(_a, &Abstract3DController::horizontalAspectRatioChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(QAbstract3DGraph::OptimizationHints )>(_a, &Abstract3DController::optimizationHintsChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(bool )>(_a, &Abstract3DController::polarChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(float )>(_a, &Abstract3DController::radialLabelOffsetChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(bool )>(_a, &Abstract3DController::reflectionChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(qreal )>(_a, &Abstract3DController::reflectivityChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(const QLocale & )>(_a, &Abstract3DController::localeChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(const QVector3D & )>(_a, &Abstract3DController::queriedGraphPositionChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Abstract3DController::*)(qreal )>(_a, &Abstract3DController::marginChanged, 21))
            return;
    }
}

const QMetaObject *Abstract3DController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Abstract3DController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20Abstract3DControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Abstract3DController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    return _id;
}

// SIGNAL 0
void Abstract3DController::shadowQualityChanged(QAbstract3DGraph::ShadowQuality _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Abstract3DController::activeInputHandlerChanged(QAbstract3DInputHandler * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Abstract3DController::activeThemeChanged(Q3DTheme * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Abstract3DController::selectionModeChanged(QAbstract3DGraph::SelectionFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Abstract3DController::needRender()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Abstract3DController::axisXChanged(QAbstract3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Abstract3DController::axisYChanged(QAbstract3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Abstract3DController::axisZChanged(QAbstract3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Abstract3DController::elementSelected(QAbstract3DGraph::ElementType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Abstract3DController::measureFpsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Abstract3DController::currentFpsChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Abstract3DController::orthoProjectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Abstract3DController::aspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Abstract3DController::horizontalAspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Abstract3DController::optimizationHintsChanged(QAbstract3DGraph::OptimizationHints _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Abstract3DController::polarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Abstract3DController::radialLabelOffsetChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Abstract3DController::reflectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Abstract3DController::reflectivityChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Abstract3DController::localeChanged(const QLocale & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Abstract3DController::queriedGraphPositionChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Abstract3DController::marginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
