/****************************************************************************
** Meta object code from reading C++ file 'bars3drenderer_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/engine/bars3drenderer_p.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtDataVisualization/qbar3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bars3drenderer_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14Bars3DRendererE_t {};
} // unnamed namespace

template <> constexpr inline auto Bars3DRenderer::qt_create_metaobjectdata<qt_meta_tag_ZN14Bars3DRendererE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Bars3DRenderer",
        "updateMultiSeriesScaling",
        "",
        "uniform",
        "updateBarSpecs",
        "GLfloat",
        "thicknessRatio",
        "QSizeF",
        "spacing",
        "relative",
        "updateBarSeriesMargin",
        "margin",
        "updateSlicingActive",
        "isSlicing",
        "updateSelectedBar",
        "QPoint",
        "position",
        "QBar3DSeries*",
        "series",
        "clickedPosition",
        "resetClickedStatus",
        "updateAxisRange",
        "QAbstract3DAxis::AxisOrientation",
        "orientation",
        "min",
        "max",
        "updateAxisReversed",
        "enable"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'updateMultiSeriesScaling'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'updateBarSpecs'
        QtMocHelpers::SlotData<void(GLfloat, const QSizeF &, bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'updateBarSpecs'
        QtMocHelpers::SlotData<void(GLfloat, const QSizeF &)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'updateBarSpecs'
        QtMocHelpers::SlotData<void(GLfloat)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'updateBarSpecs'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'updateBarSeriesMargin'
        QtMocHelpers::SlotData<void(const QSizeF &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 11 },
        }}),
        // Slot 'updateSlicingActive'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'updateSelectedBar'
        QtMocHelpers::SlotData<void(const QPoint &, QBar3DSeries *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'clickedPosition'
        QtMocHelpers::SlotData<QPoint() const>(19, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'resetClickedStatus'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateAxisRange'
        QtMocHelpers::SlotData<void(QAbstract3DAxis::AxisOrientation, float, float)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { QMetaType::Float, 24 }, { QMetaType::Float, 25 },
        }}),
        // Slot 'updateAxisReversed'
        QtMocHelpers::SlotData<void(QAbstract3DAxis::AxisOrientation, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { QMetaType::Bool, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Bars3DRenderer, qt_meta_tag_ZN14Bars3DRendererE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Bars3DRenderer::staticMetaObject = { {
    QMetaObject::SuperData::link<Abstract3DRenderer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14Bars3DRendererE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14Bars3DRendererE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14Bars3DRendererE_t>.metaTypes,
    nullptr
} };

void Bars3DRenderer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Bars3DRenderer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateMultiSeriesScaling((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->updateBarSpecs((*reinterpret_cast<std::add_pointer_t<GLfloat>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->updateBarSpecs((*reinterpret_cast<std::add_pointer_t<GLfloat>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[2]))); break;
        case 3: _t->updateBarSpecs((*reinterpret_cast<std::add_pointer_t<GLfloat>>(_a[1]))); break;
        case 4: _t->updateBarSpecs(); break;
        case 5: _t->updateBarSeriesMargin((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        case 6: _t->updateSlicingActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->updateSelectedBar((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[2]))); break;
        case 8: { QPoint _r = _t->clickedPosition();
            if (_a[0]) *reinterpret_cast<QPoint*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->resetClickedStatus(); break;
        case 10: _t->updateAxisRange((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis::AxisOrientation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 11: _t->updateAxisReversed((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis::AxisOrientation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *Bars3DRenderer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Bars3DRenderer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14Bars3DRendererE_t>.strings))
        return static_cast<void*>(this);
    return Abstract3DRenderer::qt_metacast(_clname);
}

int Bars3DRenderer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Abstract3DRenderer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
