/****************************************************************************
** Meta object code from reading C++ file 'abstractdeclarative_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualizationqml/abstractdeclarative_p.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtDataVisualization/qbar3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractdeclarative_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19AbstractDeclarativeE_t {};
} // unnamed namespace

template <> constexpr inline auto AbstractDeclarative::qt_create_metaobjectdata<qt_meta_tag_ZN19AbstractDeclarativeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AbstractDeclarative",
        "QML.Element",
        "AbstractGraph3D",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: AbstractGraph3D.",
        "selectionModeChanged",
        "",
        "AbstractDeclarative::SelectionFlags",
        "mode",
        "shadowQualityChanged",
        "AbstractDeclarative::ShadowQuality",
        "quality",
        "shadowsSupportedChanged",
        "supported",
        "msaaSamplesChanged",
        "samples",
        "sceneChanged",
        "Q3DScene*",
        "scene",
        "inputHandlerChanged",
        "QAbstract3DInputHandler*",
        "inputHandler",
        "themeChanged",
        "Q3DTheme*",
        "theme",
        "renderingModeChanged",
        "AbstractDeclarative::RenderingMode",
        "measureFpsChanged",
        "enabled",
        "currentFpsChanged",
        "fps",
        "selectedElementChanged",
        "AbstractDeclarative::ElementType",
        "type",
        "orthoProjectionChanged",
        "aspectRatioChanged",
        "ratio",
        "optimizationHintsChanged",
        "AbstractDeclarative::OptimizationHints",
        "hints",
        "polarChanged",
        "radialLabelOffsetChanged",
        "offset",
        "horizontalAspectRatioChanged",
        "reflectionChanged",
        "reflectivityChanged",
        "reflectivity",
        "localeChanged",
        "locale",
        "queriedGraphPositionChanged",
        "QVector3D",
        "data",
        "marginChanged",
        "margin",
        "handleAxisXChanged",
        "QAbstract3DAxis*",
        "axis",
        "handleAxisYChanged",
        "handleAxisZChanged",
        "windowDestroyed",
        "obj",
        "destroyContext",
        "clearSelection",
        "hasSeries",
        "QAbstract3DSeries*",
        "series",
        "addCustomItem",
        "QCustom3DItem*",
        "item",
        "removeCustomItems",
        "removeCustomItem",
        "removeCustomItemAt",
        "position",
        "releaseCustomItem",
        "selectedLabelIndex",
        "selectedAxis",
        "selectedCustomItemIndex",
        "selectedCustomItem",
        "selectionMode",
        "SelectionFlags",
        "shadowQuality",
        "ShadowQuality",
        "shadowsSupported",
        "msaaSamples",
        "Declarative3DScene*",
        "renderingMode",
        "RenderingMode",
        "measureFps",
        "currentFps",
        "customItemList",
        "QQmlListProperty<QCustom3DItem>",
        "orthoProjection",
        "selectedElement",
        "ElementType",
        "aspectRatio",
        "optimizationHints",
        "OptimizationHints",
        "polar",
        "radialLabelOffset",
        "horizontalAspectRatio",
        "reflection",
        "queriedGraphPosition",
        "SelectionFlag",
        "SelectionNone",
        "SelectionItem",
        "SelectionRow",
        "SelectionItemAndRow",
        "SelectionColumn",
        "SelectionItemAndColumn",
        "SelectionRowAndColumn",
        "SelectionItemRowAndColumn",
        "SelectionSlice",
        "SelectionMultiSeries",
        "ShadowQualityNone",
        "ShadowQualityLow",
        "ShadowQualityMedium",
        "ShadowQualityHigh",
        "ShadowQualitySoftLow",
        "ShadowQualitySoftMedium",
        "ShadowQualitySoftHigh",
        "ElementNone",
        "ElementSeries",
        "ElementAxisXLabel",
        "ElementAxisYLabel",
        "ElementAxisZLabel",
        "ElementCustomItem",
        "RenderDirectToBackground",
        "RenderDirectToBackground_NoClear",
        "RenderIndirect",
        "OptimizationHint",
        "OptimizationDefault",
        "OptimizationStatic"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectionModeChanged'
        QtMocHelpers::SignalData<void(AbstractDeclarative::SelectionFlags)>(9, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'shadowQualityChanged'
        QtMocHelpers::SignalData<void(AbstractDeclarative::ShadowQuality)>(13, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'shadowsSupportedChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'msaaSamplesChanged'
        QtMocHelpers::SignalData<void(int)>(18, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Signal 'sceneChanged'
        QtMocHelpers::SignalData<void(Q3DScene *)>(20, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'inputHandlerChanged'
        QtMocHelpers::SignalData<void(QAbstract3DInputHandler *)>(23, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'themeChanged'
        QtMocHelpers::SignalData<void(Q3DTheme *)>(26, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'renderingModeChanged'
        QtMocHelpers::SignalData<void(AbstractDeclarative::RenderingMode)>(29, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 12 },
        }}),
        // Signal 'measureFpsChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(31, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Signal 'currentFpsChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(33, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QReal, 34 },
        }}),
        // Signal 'selectedElementChanged'
        QtMocHelpers::RevisionedSignalData<void(AbstractDeclarative::ElementType)>(35, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'orthoProjectionChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(38, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Signal 'aspectRatioChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(39, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
        // Signal 'optimizationHintsChanged'
        QtMocHelpers::RevisionedSignalData<void(AbstractDeclarative::OptimizationHints)>(41, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Signal 'polarChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(44, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Signal 'radialLabelOffsetChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(45, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::Float, 46 },
        }}),
        // Signal 'horizontalAspectRatioChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(47, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
        // Signal 'reflectionChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(48, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Signal 'reflectivityChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(49, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::QReal, 50 },
        }}),
        // Signal 'localeChanged'
        QtMocHelpers::RevisionedSignalData<void(const QLocale &)>(51, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::QLocale, 52 },
        }}),
        // Signal 'queriedGraphPositionChanged'
        QtMocHelpers::RevisionedSignalData<void(const QVector3D &)>(53, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Signal 'marginChanged'
        QtMocHelpers::RevisionedSignalData<void(qreal)>(56, 10, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::QReal, 57 },
        }}),
        // Slot 'handleAxisXChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(58, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'handleAxisYChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(61, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'handleAxisZChanged'
        QtMocHelpers::SlotData<void(QAbstract3DAxis *)>(62, 10, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'windowDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(63, 10, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 64 },
        }}),
        // Slot 'destroyContext'
        QtMocHelpers::SlotData<void()>(65, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'clearSelection'
        QtMocHelpers::MethodData<void()>(66, 10, QMC::AccessPublic, QMetaType::Void),
        // Method 'hasSeries'
        QtMocHelpers::RevisionedMethodData<bool(QAbstract3DSeries *)>(67, 10, QMC::AccessPublic, 0x603, QMetaType::Bool, {{
            { 0x80000000 | 68, 69 },
        }}),
        // Method 'addCustomItem'
        QtMocHelpers::RevisionedMethodData<int(QCustom3DItem *)>(70, 10, QMC::AccessPublic, 0x101, QMetaType::Int, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Method 'removeCustomItems'
        QtMocHelpers::RevisionedMethodData<void()>(73, 10, QMC::AccessPublic, 0x101, QMetaType::Void),
        // Method 'removeCustomItem'
        QtMocHelpers::RevisionedMethodData<void(QCustom3DItem *)>(74, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Method 'removeCustomItemAt'
        QtMocHelpers::RevisionedMethodData<void(const QVector3D &)>(75, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 54, 76 },
        }}),
        // Method 'releaseCustomItem'
        QtMocHelpers::RevisionedMethodData<void(QCustom3DItem *)>(77, 10, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Method 'selectedLabelIndex'
        QtMocHelpers::RevisionedMethodData<int() const>(78, 10, QMC::AccessPublic, 0x101, QMetaType::Int),
        // Method 'selectedAxis'
        QtMocHelpers::RevisionedMethodData<QAbstract3DAxis *() const>(79, 10, QMC::AccessPublic, 0x101, 0x80000000 | 59),
        // Method 'selectedCustomItemIndex'
        QtMocHelpers::RevisionedMethodData<int() const>(80, 10, QMC::AccessPublic, 0x101, QMetaType::Int),
        // Method 'selectedCustomItem'
        QtMocHelpers::RevisionedMethodData<QCustom3DItem *() const>(81, 10, QMC::AccessPublic, 0x101, 0x80000000 | 71),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'selectionMode'
        QtMocHelpers::PropertyData<SelectionFlags>(82, 0x80000000 | 83, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'shadowQuality'
        QtMocHelpers::PropertyData<enum ShadowQuality>(84, 0x80000000 | 85, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'shadowsSupported'
        QtMocHelpers::PropertyData<bool>(86, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'msaaSamples'
        QtMocHelpers::PropertyData<int>(87, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'scene'
        QtMocHelpers::PropertyData<Declarative3DScene*>(22, 0x80000000 | 88, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'inputHandler'
        QtMocHelpers::PropertyData<QAbstract3DInputHandler*>(25, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'theme'
        QtMocHelpers::PropertyData<Q3DTheme*>(28, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'renderingMode'
        QtMocHelpers::PropertyData<enum RenderingMode>(89, 0x80000000 | 90, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'measureFps'
        QtMocHelpers::PropertyData<bool>(91, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8, 0x101),
        // property 'currentFps'
        QtMocHelpers::PropertyData<qreal>(92, QMetaType::QReal, QMC::DefaultPropertyFlags, 9, 0x101),
        // property 'customItemList'
        QtMocHelpers::PropertyData<QQmlListProperty<QCustom3DItem>>(93, 0x80000000 | 94, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, -1, 0x101),
        // property 'orthoProjection'
        QtMocHelpers::PropertyData<bool>(95, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11, 0x101),
        // property 'selectedElement'
        QtMocHelpers::PropertyData<enum ElementType>(96, 0x80000000 | 97, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 10, 0x101),
        // property 'aspectRatio'
        QtMocHelpers::PropertyData<qreal>(98, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12, 0x101),
        // property 'optimizationHints'
        QtMocHelpers::PropertyData<OptimizationHints>(99, 0x80000000 | 100, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 13, 0x101),
        // property 'polar'
        QtMocHelpers::PropertyData<bool>(101, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14, 0x102),
        // property 'radialLabelOffset'
        QtMocHelpers::PropertyData<float>(102, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15, 0x102),
        // property 'horizontalAspectRatio'
        QtMocHelpers::PropertyData<qreal>(103, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16, 0x102),
        // property 'reflection'
        QtMocHelpers::PropertyData<bool>(104, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17, 0x102),
        // property 'reflectivity'
        QtMocHelpers::PropertyData<qreal>(50, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18, 0x102),
        // property 'locale'
        QtMocHelpers::PropertyData<QLocale>(52, QMetaType::QLocale, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19, 0x102),
        // property 'queriedGraphPosition'
        QtMocHelpers::PropertyData<QVector3D>(105, 0x80000000 | 54, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 20, 0x102),
        // property 'margin'
        QtMocHelpers::PropertyData<qreal>(57, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21, 0x102),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelectionFlag'
        QtMocHelpers::EnumData<enum SelectionFlag>(106, 106, QMC::EnumFlags{}).add({
            {  107, SelectionFlag::SelectionNone },
            {  108, SelectionFlag::SelectionItem },
            {  109, SelectionFlag::SelectionRow },
            {  110, SelectionFlag::SelectionItemAndRow },
            {  111, SelectionFlag::SelectionColumn },
            {  112, SelectionFlag::SelectionItemAndColumn },
            {  113, SelectionFlag::SelectionRowAndColumn },
            {  114, SelectionFlag::SelectionItemRowAndColumn },
            {  115, SelectionFlag::SelectionSlice },
            {  116, SelectionFlag::SelectionMultiSeries },
        }),
        // flag 'SelectionFlags'
        QtMocHelpers::EnumData<SelectionFlags>(83, 106, QMC::EnumIsFlag).add({
            {  107, SelectionFlag::SelectionNone },
            {  108, SelectionFlag::SelectionItem },
            {  109, SelectionFlag::SelectionRow },
            {  110, SelectionFlag::SelectionItemAndRow },
            {  111, SelectionFlag::SelectionColumn },
            {  112, SelectionFlag::SelectionItemAndColumn },
            {  113, SelectionFlag::SelectionRowAndColumn },
            {  114, SelectionFlag::SelectionItemRowAndColumn },
            {  115, SelectionFlag::SelectionSlice },
            {  116, SelectionFlag::SelectionMultiSeries },
        }),
        // enum 'ShadowQuality'
        QtMocHelpers::EnumData<enum ShadowQuality>(85, 85, QMC::EnumFlags{}).add({
            {  117, ShadowQuality::ShadowQualityNone },
            {  118, ShadowQuality::ShadowQualityLow },
            {  119, ShadowQuality::ShadowQualityMedium },
            {  120, ShadowQuality::ShadowQualityHigh },
            {  121, ShadowQuality::ShadowQualitySoftLow },
            {  122, ShadowQuality::ShadowQualitySoftMedium },
            {  123, ShadowQuality::ShadowQualitySoftHigh },
        }),
        // enum 'ElementType'
        QtMocHelpers::EnumData<enum ElementType>(97, 97, QMC::EnumFlags{}).add({
            {  124, ElementType::ElementNone },
            {  125, ElementType::ElementSeries },
            {  126, ElementType::ElementAxisXLabel },
            {  127, ElementType::ElementAxisYLabel },
            {  128, ElementType::ElementAxisZLabel },
            {  129, ElementType::ElementCustomItem },
        }),
        // enum 'RenderingMode'
        QtMocHelpers::EnumData<enum RenderingMode>(90, 90, QMC::EnumFlags{}).add({
            {  130, RenderingMode::RenderDirectToBackground },
            {  131, RenderingMode::RenderDirectToBackground_NoClear },
            {  132, RenderingMode::RenderIndirect },
        }),
        // enum 'OptimizationHint'
        QtMocHelpers::EnumData<enum OptimizationHint>(133, 133, QMC::EnumFlags{}).add({
            {  134, OptimizationHint::OptimizationDefault },
            {  135, OptimizationHint::OptimizationStatic },
        }),
        // flag 'OptimizationHints'
        QtMocHelpers::EnumData<OptimizationHints>(100, 133, QMC::EnumIsFlag).add({
            {  134, OptimizationHint::OptimizationDefault },
            {  135, OptimizationHint::OptimizationStatic },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<AbstractDeclarative, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AbstractDeclarative::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AbstractDeclarativeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AbstractDeclarativeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19AbstractDeclarativeE_t>.metaTypes,
    nullptr
} };

void AbstractDeclarative::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractDeclarative *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectionModeChanged((*reinterpret_cast<std::add_pointer_t<AbstractDeclarative::SelectionFlags>>(_a[1]))); break;
        case 1: _t->shadowQualityChanged((*reinterpret_cast<std::add_pointer_t<AbstractDeclarative::ShadowQuality>>(_a[1]))); break;
        case 2: _t->shadowsSupportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->msaaSamplesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->sceneChanged((*reinterpret_cast<std::add_pointer_t<Q3DScene*>>(_a[1]))); break;
        case 5: _t->inputHandlerChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DInputHandler*>>(_a[1]))); break;
        case 6: _t->themeChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme*>>(_a[1]))); break;
        case 7: _t->renderingModeChanged((*reinterpret_cast<std::add_pointer_t<AbstractDeclarative::RenderingMode>>(_a[1]))); break;
        case 8: _t->measureFpsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->currentFpsChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->selectedElementChanged((*reinterpret_cast<std::add_pointer_t<AbstractDeclarative::ElementType>>(_a[1]))); break;
        case 11: _t->orthoProjectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->aspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 13: _t->optimizationHintsChanged((*reinterpret_cast<std::add_pointer_t<AbstractDeclarative::OptimizationHints>>(_a[1]))); break;
        case 14: _t->polarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->radialLabelOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 16: _t->horizontalAspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->reflectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->reflectivityChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 19: _t->localeChanged((*reinterpret_cast<std::add_pointer_t<QLocale>>(_a[1]))); break;
        case 20: _t->queriedGraphPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 21: _t->marginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 22: _t->handleAxisXChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 23: _t->handleAxisYChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 24: _t->handleAxisZChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DAxis*>>(_a[1]))); break;
        case 25: _t->windowDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 26: _t->destroyContext(); break;
        case 27: _t->clearSelection(); break;
        case 28: { bool _r = _t->hasSeries((*reinterpret_cast<std::add_pointer_t<QAbstract3DSeries*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { int _r = _t->addCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->removeCustomItems(); break;
        case 31: _t->removeCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1]))); break;
        case 32: _t->removeCustomItemAt((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 33: _t->releaseCustomItem((*reinterpret_cast<std::add_pointer_t<QCustom3DItem*>>(_a[1]))); break;
        case 34: { int _r = _t->selectedLabelIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 35: { QAbstract3DAxis* _r = _t->selectedAxis();
            if (_a[0]) *reinterpret_cast<QAbstract3DAxis**>(_a[0]) = std::move(_r); }  break;
        case 36: { int _r = _t->selectedCustomItemIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 37: { QCustom3DItem* _r = _t->selectedCustomItem();
            if (_a[0]) *reinterpret_cast<QCustom3DItem**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Q3DScene* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DInputHandler* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Q3DTheme* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DAxis* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DSeries* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCustom3DItem* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCustom3DItem* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCustom3DItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(AbstractDeclarative::SelectionFlags )>(_a, &AbstractDeclarative::selectionModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(AbstractDeclarative::ShadowQuality )>(_a, &AbstractDeclarative::shadowQualityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(bool )>(_a, &AbstractDeclarative::shadowsSupportedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(int )>(_a, &AbstractDeclarative::msaaSamplesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(Q3DScene * )>(_a, &AbstractDeclarative::sceneChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(QAbstract3DInputHandler * )>(_a, &AbstractDeclarative::inputHandlerChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(Q3DTheme * )>(_a, &AbstractDeclarative::themeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(AbstractDeclarative::RenderingMode )>(_a, &AbstractDeclarative::renderingModeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(bool )>(_a, &AbstractDeclarative::measureFpsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(qreal )>(_a, &AbstractDeclarative::currentFpsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(AbstractDeclarative::ElementType )>(_a, &AbstractDeclarative::selectedElementChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(bool )>(_a, &AbstractDeclarative::orthoProjectionChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(qreal )>(_a, &AbstractDeclarative::aspectRatioChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(AbstractDeclarative::OptimizationHints )>(_a, &AbstractDeclarative::optimizationHintsChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(bool )>(_a, &AbstractDeclarative::polarChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(float )>(_a, &AbstractDeclarative::radialLabelOffsetChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(qreal )>(_a, &AbstractDeclarative::horizontalAspectRatioChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(bool )>(_a, &AbstractDeclarative::reflectionChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(qreal )>(_a, &AbstractDeclarative::reflectivityChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(const QLocale & )>(_a, &AbstractDeclarative::localeChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(const QVector3D & )>(_a, &AbstractDeclarative::queriedGraphPositionChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractDeclarative::*)(qreal )>(_a, &AbstractDeclarative::marginChanged, 21))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Declarative3DScene* >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::assignFlags<SelectionFlags>(_v, _t->selectionMode()); break;
        case 1: *reinterpret_cast<enum ShadowQuality*>(_v) = _t->shadowQuality(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->shadowsSupported(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->msaaSamples(); break;
        case 4: *reinterpret_cast<Declarative3DScene**>(_v) = _t->scene(); break;
        case 5: *reinterpret_cast<QAbstract3DInputHandler**>(_v) = _t->inputHandler(); break;
        case 6: *reinterpret_cast<Q3DTheme**>(_v) = _t->theme(); break;
        case 7: *reinterpret_cast<enum RenderingMode*>(_v) = _t->renderingMode(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->measureFps(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->currentFps(); break;
        case 10: *reinterpret_cast<QQmlListProperty<QCustom3DItem>*>(_v) = _t->customItemList(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isOrthoProjection(); break;
        case 12: *reinterpret_cast<enum ElementType*>(_v) = _t->selectedElement(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->aspectRatio(); break;
        case 14: QtMocHelpers::assignFlags<OptimizationHints>(_v, _t->optimizationHints()); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isPolar(); break;
        case 16: *reinterpret_cast<float*>(_v) = _t->radialLabelOffset(); break;
        case 17: *reinterpret_cast<qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isReflection(); break;
        case 19: *reinterpret_cast<qreal*>(_v) = _t->reflectivity(); break;
        case 20: *reinterpret_cast<QLocale*>(_v) = _t->locale(); break;
        case 21: *reinterpret_cast<QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 22: *reinterpret_cast<qreal*>(_v) = _t->margin(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSelectionMode(*reinterpret_cast<SelectionFlags*>(_v)); break;
        case 1: _t->setShadowQuality(*reinterpret_cast<enum ShadowQuality*>(_v)); break;
        case 3: _t->setMsaaSamples(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setInputHandler(*reinterpret_cast<QAbstract3DInputHandler**>(_v)); break;
        case 6: _t->setTheme(*reinterpret_cast<Q3DTheme**>(_v)); break;
        case 7: _t->setRenderingMode(*reinterpret_cast<enum RenderingMode*>(_v)); break;
        case 8: _t->setMeasureFps(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setOrthoProjection(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 14: _t->setOptimizationHints(*reinterpret_cast<OptimizationHints*>(_v)); break;
        case 15: _t->setPolar(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setRadialLabelOffset(*reinterpret_cast<float*>(_v)); break;
        case 17: _t->setHorizontalAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 18: _t->setReflection(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setReflectivity(*reinterpret_cast<qreal*>(_v)); break;
        case 20: _t->setLocale(*reinterpret_cast<QLocale*>(_v)); break;
        case 22: _t->setMargin(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AbstractDeclarative::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AbstractDeclarative::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AbstractDeclarativeE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractDeclarativeInterface"))
        return static_cast< AbstractDeclarativeInterface*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int AbstractDeclarative::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void AbstractDeclarative::selectionModeChanged(AbstractDeclarative::SelectionFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AbstractDeclarative::shadowQualityChanged(AbstractDeclarative::ShadowQuality _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AbstractDeclarative::shadowsSupportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AbstractDeclarative::msaaSamplesChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void AbstractDeclarative::sceneChanged(Q3DScene * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AbstractDeclarative::inputHandlerChanged(QAbstract3DInputHandler * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void AbstractDeclarative::themeChanged(Q3DTheme * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void AbstractDeclarative::renderingModeChanged(AbstractDeclarative::RenderingMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void AbstractDeclarative::measureFpsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void AbstractDeclarative::currentFpsChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void AbstractDeclarative::selectedElementChanged(AbstractDeclarative::ElementType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void AbstractDeclarative::orthoProjectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void AbstractDeclarative::aspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void AbstractDeclarative::optimizationHintsChanged(AbstractDeclarative::OptimizationHints _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void AbstractDeclarative::polarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void AbstractDeclarative::radialLabelOffsetChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void AbstractDeclarative::horizontalAspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void AbstractDeclarative::reflectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void AbstractDeclarative::reflectivityChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void AbstractDeclarative::localeChanged(const QLocale & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void AbstractDeclarative::queriedGraphPositionChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void AbstractDeclarative::marginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
