/****************************************************************************
** Meta object code from reading C++ file 'qitemmodelsurfacedataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/data/qitemmodelsurfacedataproxy.h"
#include <QtDataVisualization/qsurface3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qitemmodelsurfacedataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto QItemModelSurfaceDataProxy::qt_create_metaobjectdata<qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QItemModelSurfaceDataProxy",
        "itemModelChanged",
        "",
        "const QAbstractItemModel*",
        "itemModel",
        "rowRoleChanged",
        "role",
        "columnRoleChanged",
        "xPosRoleChanged",
        "yPosRoleChanged",
        "zPosRoleChanged",
        "rowCategoriesChanged",
        "columnCategoriesChanged",
        "useModelCategoriesChanged",
        "enable",
        "autoRowCategoriesChanged",
        "autoColumnCategoriesChanged",
        "rowRolePatternChanged",
        "pattern",
        "columnRolePatternChanged",
        "xPosRolePatternChanged",
        "yPosRolePatternChanged",
        "zPosRolePatternChanged",
        "rowRoleReplaceChanged",
        "replace",
        "columnRoleReplaceChanged",
        "xPosRoleReplaceChanged",
        "yPosRoleReplaceChanged",
        "zPosRoleReplaceChanged",
        "multiMatchBehaviorChanged",
        "MultiMatchBehavior",
        "behavior",
        "rowCategoryIndex",
        "category",
        "columnCategoryIndex",
        "QAbstractItemModel*",
        "rowRole",
        "columnRole",
        "xPosRole",
        "yPosRole",
        "zPosRole",
        "rowCategories",
        "columnCategories",
        "useModelCategories",
        "autoRowCategories",
        "autoColumnCategories",
        "rowRolePattern",
        "columnRolePattern",
        "xPosRolePattern",
        "yPosRolePattern",
        "zPosRolePattern",
        "rowRoleReplace",
        "columnRoleReplace",
        "xPosRoleReplace",
        "yPosRoleReplace",
        "zPosRoleReplace",
        "multiMatchBehavior",
        "MMBFirst",
        "MMBLast",
        "MMBAverage",
        "MMBCumulativeY"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemModelChanged'
        QtMocHelpers::SignalData<void(const QAbstractItemModel *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'rowRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'columnRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'xPosRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'yPosRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'zPosRoleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'rowCategoriesChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'columnCategoriesChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'useModelCategoriesChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'autoRowCategoriesChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'autoColumnCategoriesChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'rowRolePatternChanged'
        QtMocHelpers::RevisionedSignalData<void(const QRegularExpression &)>(17, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 18 },
        }}),
        // Signal 'columnRolePatternChanged'
        QtMocHelpers::RevisionedSignalData<void(const QRegularExpression &)>(19, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 18 },
        }}),
        // Signal 'xPosRolePatternChanged'
        QtMocHelpers::RevisionedSignalData<void(const QRegularExpression &)>(20, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 18 },
        }}),
        // Signal 'yPosRolePatternChanged'
        QtMocHelpers::RevisionedSignalData<void(const QRegularExpression &)>(21, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 18 },
        }}),
        // Signal 'zPosRolePatternChanged'
        QtMocHelpers::RevisionedSignalData<void(const QRegularExpression &)>(22, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QRegularExpression, 18 },
        }}),
        // Signal 'rowRoleReplaceChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(23, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'columnRoleReplaceChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(25, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'xPosRoleReplaceChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(26, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'yPosRoleReplaceChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(27, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'zPosRoleReplaceChanged'
        QtMocHelpers::RevisionedSignalData<void(const QString &)>(28, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'multiMatchBehaviorChanged'
        QtMocHelpers::RevisionedSignalData<void(enum MultiMatchBehavior)>(29, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Method 'rowCategoryIndex'
        QtMocHelpers::MethodData<int(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 33 },
        }}),
        // Method 'columnCategoryIndex'
        QtMocHelpers::MethodData<int(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'itemModel'
        QtMocHelpers::PropertyData<QAbstractItemModel*>(4, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'rowRole'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'columnRole'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'xPosRole'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'yPosRole'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'zPosRole'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'rowCategories'
        QtMocHelpers::PropertyData<QStringList>(41, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'columnCategories'
        QtMocHelpers::PropertyData<QStringList>(42, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'useModelCategories'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'autoRowCategories'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'autoColumnCategories'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'rowRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(46, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11, 0x101),
        // property 'columnRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(47, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12, 0x101),
        // property 'xPosRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(48, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13, 0x101),
        // property 'yPosRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(49, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14, 0x101),
        // property 'zPosRolePattern'
        QtMocHelpers::PropertyData<QRegularExpression>(50, QMetaType::QRegularExpression, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15, 0x101),
        // property 'rowRoleReplace'
        QtMocHelpers::PropertyData<QString>(51, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16, 0x101),
        // property 'columnRoleReplace'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17, 0x101),
        // property 'xPosRoleReplace'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18, 0x101),
        // property 'yPosRoleReplace'
        QtMocHelpers::PropertyData<QString>(54, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19, 0x101),
        // property 'zPosRoleReplace'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20, 0x101),
        // property 'multiMatchBehavior'
        QtMocHelpers::PropertyData<enum MultiMatchBehavior>(56, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 21, 0x101),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MultiMatchBehavior'
        QtMocHelpers::EnumData<enum MultiMatchBehavior>(30, 30, QMC::EnumFlags{}).add({
            {   57, MultiMatchBehavior::MMBFirst },
            {   58, MultiMatchBehavior::MMBLast },
            {   59, MultiMatchBehavior::MMBAverage },
            {   60, MultiMatchBehavior::MMBCumulativeY },
        }),
    };
    return QtMocHelpers::metaObjectData<QItemModelSurfaceDataProxy, qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QItemModelSurfaceDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QSurfaceDataProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t>.metaTypes,
    nullptr
} };

void QItemModelSurfaceDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QItemModelSurfaceDataProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemModelChanged((*reinterpret_cast<std::add_pointer_t<const QAbstractItemModel*>>(_a[1]))); break;
        case 1: _t->rowRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->columnRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->xPosRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->yPosRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->zPosRoleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->rowCategoriesChanged(); break;
        case 7: _t->columnCategoriesChanged(); break;
        case 8: _t->useModelCategoriesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->autoRowCategoriesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->autoColumnCategoriesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->rowRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 12: _t->columnRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 13: _t->xPosRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 14: _t->yPosRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 15: _t->zPosRolePatternChanged((*reinterpret_cast<std::add_pointer_t<QRegularExpression>>(_a[1]))); break;
        case 16: _t->rowRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->columnRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->xPosRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->yPosRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->zPosRoleReplaceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->multiMatchBehaviorChanged((*reinterpret_cast<std::add_pointer_t<enum MultiMatchBehavior>>(_a[1]))); break;
        case 22: { int _r = _t->rowCategoryIndex((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 23: { int _r = _t->columnCategoryIndex((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QAbstractItemModel * )>(_a, &QItemModelSurfaceDataProxy::itemModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::rowRoleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::columnRoleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::xPosRoleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::yPosRoleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::zPosRoleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)()>(_a, &QItemModelSurfaceDataProxy::rowCategoriesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)()>(_a, &QItemModelSurfaceDataProxy::columnCategoriesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(bool )>(_a, &QItemModelSurfaceDataProxy::useModelCategoriesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(bool )>(_a, &QItemModelSurfaceDataProxy::autoRowCategoriesChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(bool )>(_a, &QItemModelSurfaceDataProxy::autoColumnCategoriesChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelSurfaceDataProxy::rowRolePatternChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelSurfaceDataProxy::columnRolePatternChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelSurfaceDataProxy::xPosRolePatternChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelSurfaceDataProxy::yPosRolePatternChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QRegularExpression & )>(_a, &QItemModelSurfaceDataProxy::zPosRolePatternChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::rowRoleReplaceChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::columnRoleReplaceChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::xPosRoleReplaceChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::yPosRoleReplaceChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(const QString & )>(_a, &QItemModelSurfaceDataProxy::zPosRoleReplaceChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QItemModelSurfaceDataProxy::*)(MultiMatchBehavior )>(_a, &QItemModelSurfaceDataProxy::multiMatchBehaviorChanged, 21))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractItemModel**>(_v) = _t->itemModel(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->rowRole(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->columnRole(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->xPosRole(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->yPosRole(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->zPosRole(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->rowCategories(); break;
        case 7: *reinterpret_cast<QStringList*>(_v) = _t->columnCategories(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->useModelCategories(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->autoRowCategories(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->autoColumnCategories(); break;
        case 11: *reinterpret_cast<QRegularExpression*>(_v) = _t->rowRolePattern(); break;
        case 12: *reinterpret_cast<QRegularExpression*>(_v) = _t->columnRolePattern(); break;
        case 13: *reinterpret_cast<QRegularExpression*>(_v) = _t->xPosRolePattern(); break;
        case 14: *reinterpret_cast<QRegularExpression*>(_v) = _t->yPosRolePattern(); break;
        case 15: *reinterpret_cast<QRegularExpression*>(_v) = _t->zPosRolePattern(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->rowRoleReplace(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->columnRoleReplace(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->xPosRoleReplace(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->yPosRoleReplace(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->zPosRoleReplace(); break;
        case 21: *reinterpret_cast<enum MultiMatchBehavior*>(_v) = _t->multiMatchBehavior(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setItemModel(*reinterpret_cast<QAbstractItemModel**>(_v)); break;
        case 1: _t->setRowRole(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setColumnRole(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setXPosRole(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setYPosRole(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setZPosRole(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setRowCategories(*reinterpret_cast<QStringList*>(_v)); break;
        case 7: _t->setColumnCategories(*reinterpret_cast<QStringList*>(_v)); break;
        case 8: _t->setUseModelCategories(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setAutoRowCategories(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setAutoColumnCategories(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setRowRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 12: _t->setColumnRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 13: _t->setXPosRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 14: _t->setYPosRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 15: _t->setZPosRolePattern(*reinterpret_cast<QRegularExpression*>(_v)); break;
        case 16: _t->setRowRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 17: _t->setColumnRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 18: _t->setXPosRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 19: _t->setYPosRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 20: _t->setZPosRoleReplace(*reinterpret_cast<QString*>(_v)); break;
        case 21: _t->setMultiMatchBehavior(*reinterpret_cast<enum MultiMatchBehavior*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QItemModelSurfaceDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QItemModelSurfaceDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QItemModelSurfaceDataProxyE_t>.strings))
        return static_cast<void*>(this);
    return QSurfaceDataProxy::qt_metacast(_clname);
}

int QItemModelSurfaceDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSurfaceDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 24;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void QItemModelSurfaceDataProxy::itemModelChanged(const QAbstractItemModel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QItemModelSurfaceDataProxy::rowRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QItemModelSurfaceDataProxy::columnRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QItemModelSurfaceDataProxy::xPosRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QItemModelSurfaceDataProxy::yPosRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QItemModelSurfaceDataProxy::zPosRoleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QItemModelSurfaceDataProxy::rowCategoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QItemModelSurfaceDataProxy::columnCategoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QItemModelSurfaceDataProxy::useModelCategoriesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QItemModelSurfaceDataProxy::autoRowCategoriesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QItemModelSurfaceDataProxy::autoColumnCategoriesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QItemModelSurfaceDataProxy::rowRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QItemModelSurfaceDataProxy::columnRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QItemModelSurfaceDataProxy::xPosRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QItemModelSurfaceDataProxy::yPosRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QItemModelSurfaceDataProxy::zPosRolePatternChanged(const QRegularExpression & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QItemModelSurfaceDataProxy::rowRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QItemModelSurfaceDataProxy::columnRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QItemModelSurfaceDataProxy::xPosRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QItemModelSurfaceDataProxy::yPosRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QItemModelSurfaceDataProxy::zPosRoleReplaceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QItemModelSurfaceDataProxy::multiMatchBehaviorChanged(MultiMatchBehavior _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
