/****************************************************************************
** Meta object code from reading C++ file 'qsurface3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/data/qsurface3dseries.h"
#include <QtDataVisualization/qsurface3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsurface3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QSurface3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QSurface3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN16QSurface3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSurface3DSeries",
        "dataProxyChanged",
        "",
        "QSurfaceDataProxy*",
        "proxy",
        "selectedPointChanged",
        "QPoint",
        "position",
        "flatShadingEnabledChanged",
        "enable",
        "flatShadingSupportedChanged",
        "drawModeChanged",
        "QSurface3DSeries::DrawFlags",
        "mode",
        "textureChanged",
        "QImage",
        "image",
        "textureFileChanged",
        "filename",
        "wireframeColorChanged",
        "QColor",
        "color",
        "dataProxy",
        "selectedPoint",
        "flatShadingEnabled",
        "flatShadingSupported",
        "drawMode",
        "DrawFlags",
        "texture",
        "textureFile",
        "wireframeColor",
        "DrawFlag",
        "DrawWireframe",
        "DrawSurface",
        "DrawSurfaceAndWireframe"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataProxyChanged'
        QtMocHelpers::SignalData<void(QSurfaceDataProxy *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'selectedPointChanged'
        QtMocHelpers::SignalData<void(const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'flatShadingEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'flatShadingSupportedChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'drawModeChanged'
        QtMocHelpers::SignalData<void(QSurface3DSeries::DrawFlags)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'textureChanged'
        QtMocHelpers::SignalData<void(const QImage &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'textureFileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'wireframeColorChanged'
        QtMocHelpers::RevisionedSignalData<void(const QColor &)>(19, 2, QMC::AccessPublic, 0x603, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataProxy'
        QtMocHelpers::PropertyData<QSurfaceDataProxy*>(22, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'selectedPoint'
        QtMocHelpers::PropertyData<QPoint>(23, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'flatShadingEnabled'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'flatShadingSupported'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'drawMode'
        QtMocHelpers::PropertyData<DrawFlags>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'texture'
        QtMocHelpers::PropertyData<QImage>(28, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'textureFile'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'wireframeColor'
        QtMocHelpers::PropertyData<QColor>(30, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7, 0x603),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DrawFlag'
        QtMocHelpers::EnumData<enum DrawFlag>(31, 31, QMC::EnumFlags{}).add({
            {   32, DrawFlag::DrawWireframe },
            {   33, DrawFlag::DrawSurface },
            {   34, DrawFlag::DrawSurfaceAndWireframe },
        }),
        // flag 'DrawFlags'
        QtMocHelpers::EnumData<DrawFlags>(27, 31, QMC::EnumIsFlag).add({
            {   32, DrawFlag::DrawWireframe },
            {   33, DrawFlag::DrawSurface },
            {   34, DrawFlag::DrawSurfaceAndWireframe },
        }),
    };
    return QtMocHelpers::metaObjectData<QSurface3DSeries, qt_meta_tag_ZN16QSurface3DSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QSurface3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.metaTypes,
    nullptr
} };

void QSurface3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSurface3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataProxyChanged((*reinterpret_cast<std::add_pointer_t<QSurfaceDataProxy*>>(_a[1]))); break;
        case 1: _t->selectedPointChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->flatShadingEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flatShadingSupportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->drawModeChanged((*reinterpret_cast<std::add_pointer_t<QSurface3DSeries::DrawFlags>>(_a[1]))); break;
        case 5: _t->textureChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 6: _t->textureFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->wireframeColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSurfaceDataProxy* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QSurfaceDataProxy * )>(_a, &QSurface3DSeries::dataProxyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QPoint & )>(_a, &QSurface3DSeries::selectedPointChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(bool )>(_a, &QSurface3DSeries::flatShadingEnabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(bool )>(_a, &QSurface3DSeries::flatShadingSupportedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(QSurface3DSeries::DrawFlags )>(_a, &QSurface3DSeries::drawModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QImage & )>(_a, &QSurface3DSeries::textureChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QString & )>(_a, &QSurface3DSeries::textureFileChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QSurface3DSeries::*)(const QColor & )>(_a, &QSurface3DSeries::wireframeColorChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSurfaceDataProxy* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QSurfaceDataProxy**>(_v) = _t->dataProxy(); break;
        case 1: *reinterpret_cast<QPoint*>(_v) = _t->selectedPoint(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isFlatShadingEnabled(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isFlatShadingSupported(); break;
        case 4: QtMocHelpers::assignFlags<DrawFlags>(_v, _t->drawMode()); break;
        case 5: *reinterpret_cast<QImage*>(_v) = _t->texture(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->textureFile(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->wireframeColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataProxy(*reinterpret_cast<QSurfaceDataProxy**>(_v)); break;
        case 1: _t->setSelectedPoint(*reinterpret_cast<QPoint*>(_v)); break;
        case 2: _t->setFlatShadingEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setDrawMode(*reinterpret_cast<DrawFlags*>(_v)); break;
        case 5: _t->setTexture(*reinterpret_cast<QImage*>(_v)); break;
        case 6: _t->setTextureFile(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setWireframeColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QSurface3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSurface3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QSurface3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DSeries::qt_metacast(_clname);
}

int QSurface3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QSurface3DSeries::dataProxyChanged(QSurfaceDataProxy * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QSurface3DSeries::selectedPointChanged(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QSurface3DSeries::flatShadingEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QSurface3DSeries::flatShadingSupportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QSurface3DSeries::drawModeChanged(QSurface3DSeries::DrawFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QSurface3DSeries::textureChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QSurface3DSeries::textureFileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QSurface3DSeries::wireframeColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
