/****************************************************************************
** Meta object code from reading C++ file 'qbardataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/data/qbardataproxy.h"
#include <QtDataVisualization/qbar3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbardataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QBarDataProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto QBarDataProxy::qt_create_metaobjectdata<qt_meta_tag_ZN13QBarDataProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBarDataProxy",
        "arrayReset",
        "",
        "rowsAdded",
        "startIndex",
        "count",
        "rowsChanged",
        "rowsRemoved",
        "rowsInserted",
        "itemChanged",
        "rowIndex",
        "columnIndex",
        "rowCountChanged",
        "rowLabelsChanged",
        "columnLabelsChanged",
        "seriesChanged",
        "QBar3DSeries*",
        "series",
        "rowCount",
        "rowLabels",
        "columnLabels"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'arrayReset'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rowsAdded'
        QtMocHelpers::SignalData<void(int, int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'rowsChanged'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'rowsRemoved'
        QtMocHelpers::SignalData<void(int, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'rowsInserted'
        QtMocHelpers::SignalData<void(int, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'itemChanged'
        QtMocHelpers::SignalData<void(int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Signal 'rowCountChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'rowLabelsChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'columnLabelsChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'seriesChanged'
        QtMocHelpers::SignalData<void(QBar3DSeries *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rowCount'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags, 6),
        // property 'rowLabels'
        QtMocHelpers::PropertyData<QStringList>(19, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'columnLabels'
        QtMocHelpers::PropertyData<QStringList>(20, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'series'
        QtMocHelpers::PropertyData<QBar3DSeries*>(17, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QBarDataProxy, qt_meta_tag_ZN13QBarDataProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QBarDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractDataProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QBarDataProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QBarDataProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QBarDataProxyE_t>.metaTypes,
    nullptr
} };

void QBarDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBarDataProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->arrayReset(); break;
        case 1: _t->rowsAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->rowsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->rowsRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->rowsInserted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->itemChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->rowCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->rowLabelsChanged(); break;
        case 8: _t->columnLabelsChanged(); break;
        case 9: _t->seriesChanged((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)()>(_a, &QBarDataProxy::arrayReset, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(int , int )>(_a, &QBarDataProxy::rowsAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(int , int )>(_a, &QBarDataProxy::rowsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(int , int )>(_a, &QBarDataProxy::rowsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(int , int )>(_a, &QBarDataProxy::rowsInserted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(int , int )>(_a, &QBarDataProxy::itemChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(int )>(_a, &QBarDataProxy::rowCountChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)()>(_a, &QBarDataProxy::rowLabelsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)()>(_a, &QBarDataProxy::columnLabelsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBarDataProxy::*)(QBar3DSeries * )>(_a, &QBarDataProxy::seriesChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->rowCount(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->rowLabels(); break;
        case 2: *reinterpret_cast<QStringList*>(_v) = _t->columnLabels(); break;
        case 3: *reinterpret_cast<QBar3DSeries**>(_v) = _t->series(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setRowLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setColumnLabels(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QBarDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBarDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QBarDataProxyE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractDataProxy::qt_metacast(_clname);
}

int QBarDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QBarDataProxy::arrayReset()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QBarDataProxy::rowsAdded(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void QBarDataProxy::rowsChanged(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QBarDataProxy::rowsRemoved(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void QBarDataProxy::rowsInserted(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void QBarDataProxy::itemChanged(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QBarDataProxy::rowCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QBarDataProxy::rowLabelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QBarDataProxy::columnLabelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QBarDataProxy::seriesChanged(QBar3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
