/****************************************************************************
** Meta object code from reading C++ file 'qcustom3dvolume.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/data/qcustom3dvolume.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcustom3dvolume.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QCustom3DVolumeE_t {};
} // unnamed namespace

template <> constexpr inline auto QCustom3DVolume::qt_create_metaobjectdata<qt_meta_tag_ZN15QCustom3DVolumeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCustom3DVolume",
        "textureWidthChanged",
        "",
        "value",
        "textureHeightChanged",
        "textureDepthChanged",
        "sliceIndexXChanged",
        "sliceIndexYChanged",
        "sliceIndexZChanged",
        "colorTableChanged",
        "textureDataChanged",
        "QList<uchar>*",
        "data",
        "textureFormatChanged",
        "QImage::Format",
        "format",
        "alphaMultiplierChanged",
        "mult",
        "preserveOpacityChanged",
        "enabled",
        "useHighDefShaderChanged",
        "drawSlicesChanged",
        "drawSliceFramesChanged",
        "sliceFrameColorChanged",
        "QColor",
        "color",
        "sliceFrameWidthsChanged",
        "QVector3D",
        "values",
        "sliceFrameGapsChanged",
        "sliceFrameThicknessesChanged",
        "textureWidth",
        "textureHeight",
        "textureDepth",
        "sliceIndexX",
        "sliceIndexY",
        "sliceIndexZ",
        "colorTable",
        "QList<QRgb>",
        "textureData",
        "alphaMultiplier",
        "preserveOpacity",
        "useHighDefShader",
        "drawSlices",
        "drawSliceFrames",
        "sliceFrameColor",
        "sliceFrameWidths",
        "sliceFrameGaps",
        "sliceFrameThicknesses"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textureWidthChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'textureHeightChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'textureDepthChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'sliceIndexXChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'sliceIndexYChanged'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'sliceIndexZChanged'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'colorTableChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'textureDataChanged'
        QtMocHelpers::SignalData<void(QList<uchar> *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'textureFormatChanged'
        QtMocHelpers::SignalData<void(QImage::Format)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'alphaMultiplierChanged'
        QtMocHelpers::SignalData<void(float)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 17 },
        }}),
        // Signal 'preserveOpacityChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'useHighDefShaderChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'drawSlicesChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'drawSliceFramesChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'sliceFrameColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'sliceFrameWidthsChanged'
        QtMocHelpers::SignalData<void(const QVector3D &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'sliceFrameGapsChanged'
        QtMocHelpers::SignalData<void(const QVector3D &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'sliceFrameThicknessesChanged'
        QtMocHelpers::SignalData<void(const QVector3D &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'textureWidth'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'textureHeight'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'textureDepth'
        QtMocHelpers::PropertyData<int>(33, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'sliceIndexX'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'sliceIndexY'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'sliceIndexZ'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'colorTable'
        QtMocHelpers::PropertyData<QList<QRgb>>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'textureData'
        QtMocHelpers::PropertyData<QList<uchar>*>(39, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'alphaMultiplier'
        QtMocHelpers::PropertyData<float>(40, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'preserveOpacity'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'useHighDefShader'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'drawSlices'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'drawSliceFrames'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'sliceFrameColor'
        QtMocHelpers::PropertyData<QColor>(45, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'sliceFrameWidths'
        QtMocHelpers::PropertyData<QVector3D>(46, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 15),
        // property 'sliceFrameGaps'
        QtMocHelpers::PropertyData<QVector3D>(47, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 16),
        // property 'sliceFrameThicknesses'
        QtMocHelpers::PropertyData<QVector3D>(48, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QCustom3DVolume, qt_meta_tag_ZN15QCustom3DVolumeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QCustom3DVolume::staticMetaObject = { {
    QMetaObject::SuperData::link<QCustom3DItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QCustom3DVolumeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QCustom3DVolumeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QCustom3DVolumeE_t>.metaTypes,
    nullptr
} };

void QCustom3DVolume::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QCustom3DVolume *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textureWidthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->textureHeightChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->textureDepthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->sliceIndexXChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->sliceIndexYChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->sliceIndexZChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->colorTableChanged(); break;
        case 7: _t->textureDataChanged((*reinterpret_cast<std::add_pointer_t<QList<uchar>*>>(_a[1]))); break;
        case 8: _t->textureFormatChanged((*reinterpret_cast<std::add_pointer_t<QImage::Format>>(_a[1]))); break;
        case 9: _t->alphaMultiplierChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 10: _t->preserveOpacityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->useHighDefShaderChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->drawSlicesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->drawSliceFramesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->sliceFrameColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->sliceFrameWidthsChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 16: _t->sliceFrameGapsChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 17: _t->sliceFrameThicknessesChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(int )>(_a, &QCustom3DVolume::textureWidthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(int )>(_a, &QCustom3DVolume::textureHeightChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(int )>(_a, &QCustom3DVolume::textureDepthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(int )>(_a, &QCustom3DVolume::sliceIndexXChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(int )>(_a, &QCustom3DVolume::sliceIndexYChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(int )>(_a, &QCustom3DVolume::sliceIndexZChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)()>(_a, &QCustom3DVolume::colorTableChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(QList<uchar> * )>(_a, &QCustom3DVolume::textureDataChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(QImage::Format )>(_a, &QCustom3DVolume::textureFormatChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(float )>(_a, &QCustom3DVolume::alphaMultiplierChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(bool )>(_a, &QCustom3DVolume::preserveOpacityChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(bool )>(_a, &QCustom3DVolume::useHighDefShaderChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(bool )>(_a, &QCustom3DVolume::drawSlicesChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(bool )>(_a, &QCustom3DVolume::drawSliceFramesChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(const QColor & )>(_a, &QCustom3DVolume::sliceFrameColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(const QVector3D & )>(_a, &QCustom3DVolume::sliceFrameWidthsChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(const QVector3D & )>(_a, &QCustom3DVolume::sliceFrameGapsChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QCustom3DVolume::*)(const QVector3D & )>(_a, &QCustom3DVolume::sliceFrameThicknessesChanged, 17))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->textureWidth(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->textureHeight(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->textureDepth(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->sliceIndexX(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->sliceIndexY(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->sliceIndexZ(); break;
        case 6: *reinterpret_cast<QList<QRgb>*>(_v) = _t->colorTable(); break;
        case 7: *reinterpret_cast<QList<uchar>**>(_v) = _t->textureData(); break;
        case 8: *reinterpret_cast<float*>(_v) = _t->alphaMultiplier(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->preserveOpacity(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->useHighDefShader(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->drawSlices(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->drawSliceFrames(); break;
        case 13: *reinterpret_cast<QColor*>(_v) = _t->sliceFrameColor(); break;
        case 14: *reinterpret_cast<QVector3D*>(_v) = _t->sliceFrameWidths(); break;
        case 15: *reinterpret_cast<QVector3D*>(_v) = _t->sliceFrameGaps(); break;
        case 16: *reinterpret_cast<QVector3D*>(_v) = _t->sliceFrameThicknesses(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextureWidth(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setTextureHeight(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setTextureDepth(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setSliceIndexX(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setSliceIndexY(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setSliceIndexZ(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setColorTable(*reinterpret_cast<QList<QRgb>*>(_v)); break;
        case 7: _t->setTextureData(*reinterpret_cast<QList<uchar>**>(_v)); break;
        case 8: _t->setAlphaMultiplier(*reinterpret_cast<float*>(_v)); break;
        case 9: _t->setPreserveOpacity(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setUseHighDefShader(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setDrawSlices(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setDrawSliceFrames(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setSliceFrameColor(*reinterpret_cast<QColor*>(_v)); break;
        case 14: _t->setSliceFrameWidths(*reinterpret_cast<QVector3D*>(_v)); break;
        case 15: _t->setSliceFrameGaps(*reinterpret_cast<QVector3D*>(_v)); break;
        case 16: _t->setSliceFrameThicknesses(*reinterpret_cast<QVector3D*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QCustom3DVolume::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCustom3DVolume::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QCustom3DVolumeE_t>.strings))
        return static_cast<void*>(this);
    return QCustom3DItem::qt_metacast(_clname);
}

int QCustom3DVolume::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCustom3DItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void QCustom3DVolume::textureWidthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QCustom3DVolume::textureHeightChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QCustom3DVolume::textureDepthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QCustom3DVolume::sliceIndexXChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QCustom3DVolume::sliceIndexYChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QCustom3DVolume::sliceIndexZChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QCustom3DVolume::colorTableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QCustom3DVolume::textureDataChanged(QList<uchar> * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QCustom3DVolume::textureFormatChanged(QImage::Format _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QCustom3DVolume::alphaMultiplierChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QCustom3DVolume::preserveOpacityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QCustom3DVolume::useHighDefShaderChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QCustom3DVolume::drawSlicesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QCustom3DVolume::drawSliceFramesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QCustom3DVolume::sliceFrameColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QCustom3DVolume::sliceFrameWidthsChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QCustom3DVolume::sliceFrameGapsChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QCustom3DVolume::sliceFrameThicknessesChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}
QT_WARNING_POP
