/****************************************************************************
** Meta object code from reading C++ file 'qabstract3dinputhandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/input/qabstract3dinputhandler.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3dinputhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23QAbstract3DInputHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DInputHandler::qt_create_metaobjectdata<qt_meta_tag_ZN23QAbstract3DInputHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DInputHandler",
        "positionChanged",
        "",
        "QPoint",
        "position",
        "inputViewChanged",
        "QAbstract3DInputHandler::InputView",
        "view",
        "sceneChanged",
        "Q3DScene*",
        "scene",
        "inputView",
        "InputView",
        "inputPosition",
        "InputViewNone",
        "InputViewOnPrimary",
        "InputViewOnSecondary"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void(const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'inputViewChanged'
        QtMocHelpers::SignalData<void(QAbstract3DInputHandler::InputView)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'sceneChanged'
        QtMocHelpers::SignalData<void(Q3DScene *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'inputView'
        QtMocHelpers::PropertyData<enum InputView>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'inputPosition'
        QtMocHelpers::PropertyData<QPoint>(13, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'scene'
        QtMocHelpers::PropertyData<Q3DScene*>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'InputView'
        QtMocHelpers::EnumData<enum InputView>(12, 12, QMC::EnumFlags{}).add({
            {   14, InputView::InputViewNone },
            {   15, InputView::InputViewOnPrimary },
            {   16, InputView::InputViewOnSecondary },
        }),
    };
    return QtMocHelpers::metaObjectData<QAbstract3DInputHandler, qt_meta_tag_ZN23QAbstract3DInputHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstract3DInputHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QAbstract3DInputHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QAbstract3DInputHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23QAbstract3DInputHandlerE_t>.metaTypes,
    nullptr
} };

void QAbstract3DInputHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstract3DInputHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->inputViewChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DInputHandler::InputView>>(_a[1]))); break;
        case 2: _t->sceneChanged((*reinterpret_cast<std::add_pointer_t<Q3DScene*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Q3DScene* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DInputHandler::*)(const QPoint & )>(_a, &QAbstract3DInputHandler::positionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DInputHandler::*)(QAbstract3DInputHandler::InputView )>(_a, &QAbstract3DInputHandler::inputViewChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DInputHandler::*)(Q3DScene * )>(_a, &QAbstract3DInputHandler::sceneChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DScene* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum InputView*>(_v) = _t->inputView(); break;
        case 1: *reinterpret_cast<QPoint*>(_v) = _t->inputPosition(); break;
        case 2: *reinterpret_cast<Q3DScene**>(_v) = _t->scene(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInputView(*reinterpret_cast<enum InputView*>(_v)); break;
        case 1: _t->setInputPosition(*reinterpret_cast<QPoint*>(_v)); break;
        case 2: _t->setScene(*reinterpret_cast<Q3DScene**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstract3DInputHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstract3DInputHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23QAbstract3DInputHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstract3DInputHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QAbstract3DInputHandler::positionChanged(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstract3DInputHandler::inputViewChanged(QAbstract3DInputHandler::InputView _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstract3DInputHandler::sceneChanged(Q3DScene * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
