/****************************************************************************
** Meta object code from reading C++ file 'declarativeseries_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualizationqml/declarativeseries_p.h"
#include <QtDataVisualization/qbar3dseries.h>
#include <QtDataVisualization/qscatter3dseries.h>
#include <QtDataVisualization/qsurface3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'declarativeseries_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22DeclarativeBar3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeBar3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN22DeclarativeBar3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeBar3DSeries",
        "DefaultProperty",
        "seriesChildren",
        "QML.Element",
        "Bar3DSeries",
        "QML.AddedInVersion",
        "256",
        "selectedBarChanged",
        "",
        "QPointF",
        "position",
        "baseGradientChanged",
        "ColorGradient*",
        "gradient",
        "singleHighlightGradientChanged",
        "multiHighlightGradientChanged",
        "handleBaseGradientUpdate",
        "handleSingleHighlightGradientUpdate",
        "handleMultiHighlightGradientUpdate",
        "handleRowColorUpdate",
        "QQmlListProperty<QObject>",
        "selectedBar",
        "invalidSelectionPosition",
        "baseGradient",
        "singleHighlightGradient",
        "multiHighlightGradient",
        "rowColors",
        "QQmlListProperty<DeclarativeColor>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectedBarChanged'
        QtMocHelpers::SignalData<void(const QPointF &)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'baseGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(15, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'handleBaseGradientUpdate'
        QtMocHelpers::SlotData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSingleHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleMultiHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleRowColorUpdate'
        QtMocHelpers::SlotData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'selectedBar'
        QtMocHelpers::PropertyData<QPointF>(21, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'invalidSelectionPosition'
        QtMocHelpers::PropertyData<QPointF>(22, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'baseGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(23, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(24, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(25, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'rowColors'
        QtMocHelpers::PropertyData<QQmlListProperty<DeclarativeColor>>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, -1, 0x603),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeBar3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeBar3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QBar3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22DeclarativeBar3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22DeclarativeBar3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22DeclarativeBar3DSeriesE_t>.metaTypes,
    nullptr
} };

void DeclarativeBar3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeBar3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectedBarChanged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 1: _t->baseGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 2: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 3: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 4: _t->handleBaseGradientUpdate(); break;
        case 5: _t->handleSingleHighlightGradientUpdate(); break;
        case 6: _t->handleMultiHighlightGradientUpdate(); break;
        case 7: _t->handleRowColorUpdate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBar3DSeries::*)(const QPointF & )>(_a, &DeclarativeBar3DSeries::selectedBarChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBar3DSeries::*)(ColorGradient * )>(_a, &DeclarativeBar3DSeries::baseGradientChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBar3DSeries::*)(ColorGradient * )>(_a, &DeclarativeBar3DSeries::singleHighlightGradientChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeBar3DSeries::*)(ColorGradient * )>(_a, &DeclarativeBar3DSeries::multiHighlightGradientChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ColorGradient* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 1: *reinterpret_cast<QPointF*>(_v) = _t->selectedBar(); break;
        case 2: *reinterpret_cast<QPointF*>(_v) = _t->invalidSelectionPosition(); break;
        case 3: *reinterpret_cast<ColorGradient**>(_v) = _t->baseGradient(); break;
        case 4: *reinterpret_cast<ColorGradient**>(_v) = _t->singleHighlightGradient(); break;
        case 5: *reinterpret_cast<ColorGradient**>(_v) = _t->multiHighlightGradient(); break;
        case 6: *reinterpret_cast<QQmlListProperty<DeclarativeColor>*>(_v) = _t->rowColors(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSelectedBar(*reinterpret_cast<QPointF*>(_v)); break;
        case 3: _t->setBaseGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        case 4: _t->setSingleHighlightGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        case 5: _t->setMultiHighlightGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeBar3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeBar3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22DeclarativeBar3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QBar3DSeries::qt_metacast(_clname);
}

int DeclarativeBar3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QBar3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeBar3DSeries::selectedBarChanged(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DeclarativeBar3DSeries::baseGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeBar3DSeries::singleHighlightGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeclarativeBar3DSeries::multiHighlightGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN26DeclarativeScatter3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeScatter3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN26DeclarativeScatter3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeScatter3DSeries",
        "DefaultProperty",
        "seriesChildren",
        "QML.Element",
        "Scatter3DSeries",
        "QML.AddedInVersion",
        "256",
        "baseGradientChanged",
        "",
        "ColorGradient*",
        "gradient",
        "singleHighlightGradientChanged",
        "multiHighlightGradientChanged",
        "handleBaseGradientUpdate",
        "handleSingleHighlightGradientUpdate",
        "handleMultiHighlightGradientUpdate",
        "QQmlListProperty<QObject>",
        "baseGradient",
        "singleHighlightGradient",
        "multiHighlightGradient",
        "invalidSelectionIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'baseGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'handleBaseGradientUpdate'
        QtMocHelpers::SlotData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSingleHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleMultiHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'baseGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(17, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(18, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(19, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'invalidSelectionIndex'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeScatter3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeScatter3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QScatter3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DeclarativeScatter3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DeclarativeScatter3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26DeclarativeScatter3DSeriesE_t>.metaTypes,
    nullptr
} };

void DeclarativeScatter3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeScatter3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->baseGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 1: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 2: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 3: _t->handleBaseGradientUpdate(); break;
        case 4: _t->handleSingleHighlightGradientUpdate(); break;
        case 5: _t->handleMultiHighlightGradientUpdate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeScatter3DSeries::*)(ColorGradient * )>(_a, &DeclarativeScatter3DSeries::baseGradientChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeScatter3DSeries::*)(ColorGradient * )>(_a, &DeclarativeScatter3DSeries::singleHighlightGradientChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeScatter3DSeries::*)(ColorGradient * )>(_a, &DeclarativeScatter3DSeries::multiHighlightGradientChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ColorGradient* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 1: *reinterpret_cast<ColorGradient**>(_v) = _t->baseGradient(); break;
        case 2: *reinterpret_cast<ColorGradient**>(_v) = _t->singleHighlightGradient(); break;
        case 3: *reinterpret_cast<ColorGradient**>(_v) = _t->multiHighlightGradient(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->invalidSelectionIndex(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setBaseGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        case 2: _t->setSingleHighlightGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        case 3: _t->setMultiHighlightGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeScatter3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeScatter3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DeclarativeScatter3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QScatter3DSeries::qt_metacast(_clname);
}

int DeclarativeScatter3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScatter3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeScatter3DSeries::baseGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DeclarativeScatter3DSeries::singleHighlightGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeScatter3DSeries::multiHighlightGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN26DeclarativeSurface3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeclarativeSurface3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN26DeclarativeSurface3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeclarativeSurface3DSeries",
        "DefaultProperty",
        "seriesChildren",
        "QML.Element",
        "Surface3DSeries",
        "QML.AddedInVersion",
        "256",
        "selectedPointChanged",
        "",
        "QPointF",
        "position",
        "baseGradientChanged",
        "ColorGradient*",
        "gradient",
        "singleHighlightGradientChanged",
        "multiHighlightGradientChanged",
        "handleBaseGradientUpdate",
        "handleSingleHighlightGradientUpdate",
        "handleMultiHighlightGradientUpdate",
        "QQmlListProperty<QObject>",
        "selectedPoint",
        "invalidSelectionPosition",
        "baseGradient",
        "singleHighlightGradient",
        "multiHighlightGradient"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectedPointChanged'
        QtMocHelpers::SignalData<void(const QPointF &)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'baseGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(ColorGradient *)>(15, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'handleBaseGradientUpdate'
        QtMocHelpers::SlotData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleSingleHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleMultiHighlightGradientUpdate'
        QtMocHelpers::SlotData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'seriesChildren'
        QtMocHelpers::PropertyData<QQmlListProperty<QObject>>(2, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'selectedPoint'
        QtMocHelpers::PropertyData<QPointF>(20, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'invalidSelectionPosition'
        QtMocHelpers::PropertyData<QPointF>(21, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'baseGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(22, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(23, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<ColorGradient*>(24, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<DeclarativeSurface3DSeries, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeclarativeSurface3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QSurface3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DeclarativeSurface3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DeclarativeSurface3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26DeclarativeSurface3DSeriesE_t>.metaTypes,
    nullptr
} };

void DeclarativeSurface3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeclarativeSurface3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectedPointChanged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 1: _t->baseGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 2: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 3: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<ColorGradient*>>(_a[1]))); break;
        case 4: _t->handleBaseGradientUpdate(); break;
        case 5: _t->handleSingleHighlightGradientUpdate(); break;
        case 6: _t->handleMultiHighlightGradientUpdate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ColorGradient* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeclarativeSurface3DSeries::*)(const QPointF & )>(_a, &DeclarativeSurface3DSeries::selectedPointChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeSurface3DSeries::*)(ColorGradient * )>(_a, &DeclarativeSurface3DSeries::baseGradientChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeSurface3DSeries::*)(ColorGradient * )>(_a, &DeclarativeSurface3DSeries::singleHighlightGradientChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeclarativeSurface3DSeries::*)(ColorGradient * )>(_a, &DeclarativeSurface3DSeries::multiHighlightGradientChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ColorGradient* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlListProperty<QObject>*>(_v) = _t->seriesChildren(); break;
        case 1: *reinterpret_cast<QPointF*>(_v) = _t->selectedPoint(); break;
        case 2: *reinterpret_cast<QPointF*>(_v) = _t->invalidSelectionPosition(); break;
        case 3: *reinterpret_cast<ColorGradient**>(_v) = _t->baseGradient(); break;
        case 4: *reinterpret_cast<ColorGradient**>(_v) = _t->singleHighlightGradient(); break;
        case 5: *reinterpret_cast<ColorGradient**>(_v) = _t->multiHighlightGradient(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSelectedPoint(*reinterpret_cast<QPointF*>(_v)); break;
        case 3: _t->setBaseGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        case 4: _t->setSingleHighlightGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        case 5: _t->setMultiHighlightGradient(*reinterpret_cast<ColorGradient**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DeclarativeSurface3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeclarativeSurface3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26DeclarativeSurface3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QSurface3DSeries::qt_metacast(_clname);
}

int DeclarativeSurface3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSurface3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DeclarativeSurface3DSeries::selectedPointChanged(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DeclarativeSurface3DSeries::baseGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DeclarativeSurface3DSeries::singleHighlightGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DeclarativeSurface3DSeries::multiHighlightGradientChanged(ColorGradient * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
