/****************************************************************************
** Meta object code from reading C++ file 'qvalue3daxis.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/axis/qvalue3daxis.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qvalue3daxis.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12QValue3DAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto QValue3DAxis::qt_create_metaobjectdata<qt_meta_tag_ZN12QValue3DAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QValue3DAxis",
        "segmentCountChanged",
        "",
        "count",
        "subSegmentCountChanged",
        "labelFormatChanged",
        "format",
        "formatterChanged",
        "QValue3DAxisFormatter*",
        "formatter",
        "reversedChanged",
        "enable",
        "segmentCount",
        "subSegmentCount",
        "labelFormat",
        "reversed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'segmentCountChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'subSegmentCountChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'labelFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'formatterChanged'
        QtMocHelpers::RevisionedSignalData<void(QValue3DAxisFormatter *)>(7, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'reversedChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(10, 2, QMC::AccessPublic, 0x101, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'segmentCount'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'subSegmentCount'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'labelFormat'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'formatter'
        QtMocHelpers::PropertyData<QValue3DAxisFormatter*>(9, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3, 0x101),
        // property 'reversed'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4, 0x101),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QValue3DAxis, qt_meta_tag_ZN12QValue3DAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QValue3DAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DAxis::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QValue3DAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QValue3DAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QValue3DAxisE_t>.metaTypes,
    nullptr
} };

void QValue3DAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QValue3DAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->segmentCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->subSegmentCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->labelFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->formatterChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxisFormatter*>>(_a[1]))); break;
        case 4: _t->reversedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxisFormatter* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QValue3DAxis::*)(int )>(_a, &QValue3DAxis::segmentCountChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QValue3DAxis::*)(int )>(_a, &QValue3DAxis::subSegmentCountChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QValue3DAxis::*)(const QString & )>(_a, &QValue3DAxis::labelFormatChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QValue3DAxis::*)(QValue3DAxisFormatter * )>(_a, &QValue3DAxis::formatterChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QValue3DAxis::*)(bool )>(_a, &QValue3DAxis::reversedChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxisFormatter* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->segmentCount(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->subSegmentCount(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->labelFormat(); break;
        case 3: *reinterpret_cast<QValue3DAxisFormatter**>(_v) = _t->formatter(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->reversed(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSegmentCount(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setSubSegmentCount(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setLabelFormat(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setFormatter(*reinterpret_cast<QValue3DAxisFormatter**>(_v)); break;
        case 4: _t->setReversed(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QValue3DAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QValue3DAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QValue3DAxisE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DAxis::qt_metacast(_clname);
}

int QValue3DAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DAxis::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QValue3DAxis::segmentCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QValue3DAxis::subSegmentCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QValue3DAxis::labelFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QValue3DAxis::formatterChanged(QValue3DAxisFormatter * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QValue3DAxis::reversedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
