/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/abstractdeclarative_p.h>)
#  include <private/abstractdeclarative_p.h>
#endif
#if __has_include(<private/colorgradient_p.h>)
#  include <private/colorgradient_p.h>
#endif
#if __has_include(<private/declarativebars_p.h>)
#  include <private/declarativebars_p.h>
#endif
#if __has_include(<private/declarativecolor_p.h>)
#  include <private/declarativecolor_p.h>
#endif
#if __has_include(<private/declarativescatter_p.h>)
#  include <private/declarativescatter_p.h>
#endif
#if __has_include(<private/declarativescene_p.h>)
#  include <private/declarativescene_p.h>
#endif
#if __has_include(<private/declarativeseries_p.h>)
#  include <private/declarativeseries_p.h>
#endif
#if __has_include(<private/declarativesurface_p.h>)
#  include <private/declarativesurface_p.h>
#endif
#if __has_include(<private/declarativetheme_p.h>)
#  include <private/declarativetheme_p.h>
#endif
#if __has_include(<private/foreigntypes_p.h>)
#  include <private/foreigntypes_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtDataVisualization()
{
    qmlRegisterModule("QtDataVisualization", 1, 0);
    qmlRegisterModule("QtDataVisualization", 1, 254);
    qmlRegisterModule("QtDataVisualization", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<AbstractDeclarative>("QtDataVisualization", 6);
    qmlRegisterAnonymousTypesAndRevisions<QQuickItem>("QtDataVisualization", 6);
    qmlRegisterEnum<AbstractDeclarative::SelectionFlag>("AbstractDeclarative::SelectionFlag");
    qmlRegisterEnum<AbstractDeclarative::SelectionFlags>("AbstractDeclarative::SelectionFlags");
    qmlRegisterEnum<AbstractDeclarative::SelectionFlag>("AbstractDeclarative::SelectionFlag");
    qmlRegisterEnum<AbstractDeclarative::ShadowQuality>("AbstractDeclarative::ShadowQuality");
    qmlRegisterEnum<AbstractDeclarative::ElementType>("AbstractDeclarative::ElementType");
    qmlRegisterEnum<AbstractDeclarative::RenderingMode>("AbstractDeclarative::RenderingMode");
    qmlRegisterEnum<AbstractDeclarative::OptimizationHint>("AbstractDeclarative::OptimizationHint");
    qmlRegisterEnum<AbstractDeclarative::OptimizationHints>("AbstractDeclarative::OptimizationHints");
    qmlRegisterEnum<AbstractDeclarative::OptimizationHint>("AbstractDeclarative::OptimizationHint");
    qmlRegisterTypesAndRevisions<ColorGradient>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<ColorGradientStop>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<Declarative3DScene>("QtDataVisualization", 6);
    qmlRegisterAnonymousTypesAndRevisions<Q3DScene>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeBar3DSeries>("QtDataVisualization", 6);
    qmlRegisterAnonymousTypesAndRevisions<QBar3DSeries>("QtDataVisualization", 6);
    qmlRegisterAnonymousTypesAndRevisions<QAbstract3DSeries>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeBars>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeColor>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeScatter>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeScatter3DSeries>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeSurface>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeSurface3DSeries>("QtDataVisualization", 6);
    qmlRegisterAnonymousTypesAndRevisions<QSurface3DSeries>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<DeclarativeTheme3D>("QtDataVisualization", 6);
    qmlRegisterTypesAndRevisions<Q3DCameraDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<Q3DCameraDataVisForeign *>().id();
    qmlRegisterEnum<Q3DCamera::CameraPreset>("Q3DCamera::CameraPreset");
    qmlRegisterTypesAndRevisions<Q3DInputHandlerDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<Q3DInputHandlerDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<Q3DLightDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<Q3DLightDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<Q3DObjectDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<Q3DObjectDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<Q3DSceneForeign>("QtDataVisualization", 6);
    QMetaType::fromType<Q3DSceneForeign *>().id();
    qmlRegisterTypesAndRevisions<Q3DThemeDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<Q3DThemeDataVisForeign *>().id();
    qmlRegisterEnum<Q3DTheme::ColorStyle>("Q3DTheme::ColorStyle");
    qmlRegisterEnum<Q3DTheme::Theme>("Q3DTheme::Theme");
    qmlRegisterTypesAndRevisions<QAbstract3DAxisDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QAbstract3DAxisDataVisForeign *>().id();
    qmlRegisterEnum<QAbstract3DAxis::AxisOrientation>("QAbstract3DAxis::AxisOrientation");
    qmlRegisterEnum<QAbstract3DAxis::AxisType>("QAbstract3DAxis::AxisType");
    qmlRegisterTypesAndRevisions<QAbstract3DInputHandlerDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QAbstract3DInputHandlerDataVisForeign *>().id();
    qmlRegisterEnum<QAbstract3DInputHandler::InputView>("QAbstract3DInputHandler::InputView");
    qmlRegisterTypesAndRevisions<QAbstract3DSeriesDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QAbstract3DSeriesDataVisForeign *>().id();
    qmlRegisterEnum<QAbstract3DSeries::SeriesType>("QAbstract3DSeries::SeriesType");
    qmlRegisterEnum<QAbstract3DSeries::Mesh>("QAbstract3DSeries::Mesh");
    qmlRegisterTypesAndRevisions<QAbstractDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QAbstractDataProxyDataVisForeign *>().id();
    qmlRegisterEnum<QAbstractDataProxy::DataType>("QAbstractDataProxy::DataType");
    qmlRegisterTypesAndRevisions<QAbstractItemModelDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QAbstractItemModelDataVisForeign *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    qmlRegisterTypesAndRevisions<QBar3DSeriesDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QBar3DSeriesDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QBarDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QBarDataProxyDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QCategory3DAxisDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QCategory3DAxisDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QCustom3DItemDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QCustom3DItemDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QCustom3DLabelDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QCustom3DLabelDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QCustom3DVolumeDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QCustom3DVolumeDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QHeightMapSurfaceDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QHeightMapSurfaceDataProxyDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QItemModelBarDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QItemModelBarDataProxyDataVisForeign *>().id();
    qmlRegisterEnum<QItemModelBarDataProxy::MultiMatchBehavior>("QItemModelBarDataProxy::MultiMatchBehavior");
    qmlRegisterTypesAndRevisions<QItemModelScatterDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QItemModelScatterDataProxyDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QItemModelSurfaceDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QItemModelSurfaceDataProxyDataVisForeign *>().id();
    qmlRegisterEnum<QItemModelSurfaceDataProxy::MultiMatchBehavior>("QItemModelSurfaceDataProxy::MultiMatchBehavior");
    qmlRegisterTypesAndRevisions<QLogValue3DAxisFormatterDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QLogValue3DAxisFormatterDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QScatter3DSeriesDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QScatter3DSeriesDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QScatterDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QScatterDataProxyDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QSurface3DSeriesDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QSurface3DSeriesDataVisForeign *>().id();
    qmlRegisterEnum<QSurface3DSeries::DrawFlag>("QSurface3DSeries::DrawFlag");
    qmlRegisterEnum<QSurface3DSeries::DrawFlags>("QSurface3DSeries::DrawFlags");
    qmlRegisterEnum<QSurface3DSeries::DrawFlag>("QSurface3DSeries::DrawFlag");
    qmlRegisterTypesAndRevisions<QSurfaceDataProxyDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QSurfaceDataProxyDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QTouch3DInputHandlerDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QTouch3DInputHandlerDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QValue3DAxisDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QValue3DAxisDataVisForeign *>().id();
    qmlRegisterTypesAndRevisions<QValue3DAxisFormatterDataVisForeign>("QtDataVisualization", 6);
    QMetaType::fromType<QValue3DAxisFormatterDataVisForeign *>().id();
    QT_WARNING_POP
    qmlRegisterModule("QtDataVisualization", 6, 12);
}

static const QQmlModuleRegistration qtDataVisualizationRegistration("QtDataVisualization", qml_register_types_QtDataVisualization);
