/****************************************************************************
** Meta object code from reading C++ file 'qlogvalue3daxisformatter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/axis/qlogvalue3daxisformatter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlogvalue3daxisformatter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t {};
} // unnamed namespace

template <> constexpr inline auto QLogValue3DAxisFormatter::qt_create_metaobjectdata<qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLogValue3DAxisFormatter",
        "baseChanged",
        "",
        "base",
        "autoSubGridChanged",
        "enabled",
        "showEdgeLabelsChanged",
        "autoSubGrid",
        "showEdgeLabels"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'baseChanged'
        QtMocHelpers::SignalData<void(qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 3 },
        }}),
        // Signal 'autoSubGridChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'showEdgeLabelsChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'base'
        QtMocHelpers::PropertyData<qreal>(3, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'autoSubGrid'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'showEdgeLabels'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QLogValue3DAxisFormatter, qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QLogValue3DAxisFormatter::staticMetaObject = { {
    QMetaObject::SuperData::link<QValue3DAxisFormatter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t>.metaTypes,
    nullptr
} };

void QLogValue3DAxisFormatter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLogValue3DAxisFormatter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->baseChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 1: _t->autoSubGridChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->showEdgeLabelsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QLogValue3DAxisFormatter::*)(qreal )>(_a, &QLogValue3DAxisFormatter::baseChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLogValue3DAxisFormatter::*)(bool )>(_a, &QLogValue3DAxisFormatter::autoSubGridChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QLogValue3DAxisFormatter::*)(bool )>(_a, &QLogValue3DAxisFormatter::showEdgeLabelsChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->base(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->autoSubGrid(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->showEdgeLabels(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBase(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setAutoSubGrid(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setShowEdgeLabels(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QLogValue3DAxisFormatter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLogValue3DAxisFormatter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QLogValue3DAxisFormatterE_t>.strings))
        return static_cast<void*>(this);
    return QValue3DAxisFormatter::qt_metacast(_clname);
}

int QLogValue3DAxisFormatter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QValue3DAxisFormatter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QLogValue3DAxisFormatter::baseChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QLogValue3DAxisFormatter::autoSubGridChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QLogValue3DAxisFormatter::showEdgeLabelsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
