// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include <QtDataVisualization/private/abstractdeclarativeinterface_p.h>

QT_BEGIN_NAMESPACE

AbstractDeclarativeInterface::~AbstractDeclarativeInterface()
    = default;

QT_END_NAMESPACE
