/****************************************************************************
** Meta object code from reading C++ file 'qbar3dseries.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/data/qbar3dseries.h"
#include <QtDataVisualization/qbar3dseries.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbar3dseries.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12QBar3DSeriesE_t {};
} // unnamed namespace

template <> constexpr inline auto QBar3DSeries::qt_create_metaobjectdata<qt_meta_tag_ZN12QBar3DSeriesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBar3DSeries",
        "dataProxyChanged",
        "",
        "QBarDataProxy*",
        "proxy",
        "selectedBarChanged",
        "QPoint",
        "position",
        "meshAngleChanged",
        "angle",
        "rowColorsChanged",
        "QList<QColor>",
        "rowcolors",
        "dataProxy",
        "selectedBar",
        "meshAngle",
        "rowColors"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataProxyChanged'
        QtMocHelpers::SignalData<void(QBarDataProxy *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'selectedBarChanged'
        QtMocHelpers::SignalData<void(const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'meshAngleChanged'
        QtMocHelpers::SignalData<void(float)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 9 },
        }}),
        // Signal 'rowColorsChanged'
        QtMocHelpers::RevisionedSignalData<void(const QList<QColor> &)>(10, 2, QMC::AccessPublic, 0x603, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dataProxy'
        QtMocHelpers::PropertyData<QBarDataProxy*>(13, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'selectedBar'
        QtMocHelpers::PropertyData<QPoint>(14, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'meshAngle'
        QtMocHelpers::PropertyData<float>(15, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'rowColors'
        QtMocHelpers::PropertyData<QList<QColor>>(16, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3, 0x603),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QBar3DSeries, qt_meta_tag_ZN12QBar3DSeriesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QBar3DSeries::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DSeries::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.metaTypes,
    nullptr
} };

void QBar3DSeries::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QBar3DSeries *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataProxyChanged((*reinterpret_cast<std::add_pointer_t<QBarDataProxy*>>(_a[1]))); break;
        case 1: _t->selectedBarChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->meshAngleChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->rowColorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBarDataProxy* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(QBarDataProxy * )>(_a, &QBar3DSeries::dataProxyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(const QPoint & )>(_a, &QBar3DSeries::selectedBarChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(float )>(_a, &QBar3DSeries::meshAngleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QBar3DSeries::*)(const QList<QColor> & )>(_a, &QBar3DSeries::rowColorsChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBarDataProxy* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QBarDataProxy**>(_v) = _t->dataProxy(); break;
        case 1: *reinterpret_cast<QPoint*>(_v) = _t->selectedBar(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->meshAngle(); break;
        case 3: *reinterpret_cast<QList<QColor>*>(_v) = _t->rowColors(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataProxy(*reinterpret_cast<QBarDataProxy**>(_v)); break;
        case 1: _t->setSelectedBar(*reinterpret_cast<QPoint*>(_v)); break;
        case 2: _t->setMeshAngle(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->setRowColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QBar3DSeries::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBar3DSeries::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QBar3DSeriesE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DSeries::qt_metacast(_clname);
}

int QBar3DSeries::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DSeries::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QBar3DSeries::dataProxyChanged(QBarDataProxy * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QBar3DSeries::selectedBarChanged(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QBar3DSeries::meshAngleChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QBar3DSeries::rowColorsChanged(const QList<QColor> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
