/****************************************************************************
** Meta object code from reading C++ file 'q3dtheme.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/theme/q3dtheme.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dtheme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8Q3DThemeE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DTheme::qt_create_metaobjectdata<qt_meta_tag_ZN8Q3DThemeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DTheme",
        "typeChanged",
        "",
        "Q3DTheme::Theme",
        "themeType",
        "baseColorsChanged",
        "QList<QColor>",
        "colors",
        "backgroundColorChanged",
        "QColor",
        "color",
        "windowColorChanged",
        "labelTextColorChanged",
        "labelBackgroundColorChanged",
        "gridLineColorChanged",
        "singleHighlightColorChanged",
        "multiHighlightColorChanged",
        "lightColorChanged",
        "baseGradientsChanged",
        "QList<QLinearGradient>",
        "gradients",
        "singleHighlightGradientChanged",
        "QLinearGradient",
        "gradient",
        "multiHighlightGradientChanged",
        "lightStrengthChanged",
        "strength",
        "ambientLightStrengthChanged",
        "highlightLightStrengthChanged",
        "labelBorderEnabledChanged",
        "enabled",
        "fontChanged",
        "QFont",
        "font",
        "backgroundEnabledChanged",
        "gridEnabledChanged",
        "labelBackgroundEnabledChanged",
        "colorStyleChanged",
        "Q3DTheme::ColorStyle",
        "style",
        "type",
        "Theme",
        "baseColors",
        "backgroundColor",
        "windowColor",
        "labelTextColor",
        "labelBackgroundColor",
        "gridLineColor",
        "singleHighlightColor",
        "multiHighlightColor",
        "lightColor",
        "baseGradients",
        "singleHighlightGradient",
        "multiHighlightGradient",
        "lightStrength",
        "ambientLightStrength",
        "highlightLightStrength",
        "labelBorderEnabled",
        "backgroundEnabled",
        "gridEnabled",
        "labelBackgroundEnabled",
        "colorStyle",
        "ColorStyle",
        "ColorStyleUniform",
        "ColorStyleObjectGradient",
        "ColorStyleRangeGradient",
        "ThemeQt",
        "ThemePrimaryColors",
        "ThemeDigia",
        "ThemeStoneMoss",
        "ThemeArmyBlue",
        "ThemeRetro",
        "ThemeEbony",
        "ThemeIsabelle",
        "ThemeUserDefined"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void(Q3DTheme::Theme)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'baseColorsChanged'
        QtMocHelpers::SignalData<void(const QList<QColor> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'backgroundColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'windowColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'labelTextColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'labelBackgroundColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'gridLineColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'singleHighlightColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'multiHighlightColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'lightColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'baseGradientsChanged'
        QtMocHelpers::SignalData<void(const QList<QLinearGradient> &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'singleHighlightGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'multiHighlightGradientChanged'
        QtMocHelpers::SignalData<void(const QLinearGradient &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'lightStrengthChanged'
        QtMocHelpers::SignalData<void(float)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 26 },
        }}),
        // Signal 'ambientLightStrengthChanged'
        QtMocHelpers::SignalData<void(float)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 26 },
        }}),
        // Signal 'highlightLightStrengthChanged'
        QtMocHelpers::SignalData<void(float)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 26 },
        }}),
        // Signal 'labelBorderEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Signal 'backgroundEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Signal 'gridEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Signal 'labelBackgroundEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Signal 'colorStyleChanged'
        QtMocHelpers::SignalData<void(Q3DTheme::ColorStyle)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<enum Theme>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'baseColors'
        QtMocHelpers::PropertyData<QList<QColor>>(42, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'backgroundColor'
        QtMocHelpers::PropertyData<QColor>(43, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'windowColor'
        QtMocHelpers::PropertyData<QColor>(44, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'labelTextColor'
        QtMocHelpers::PropertyData<QColor>(45, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'labelBackgroundColor'
        QtMocHelpers::PropertyData<QColor>(46, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'gridLineColor'
        QtMocHelpers::PropertyData<QColor>(47, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'singleHighlightColor'
        QtMocHelpers::PropertyData<QColor>(48, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'multiHighlightColor'
        QtMocHelpers::PropertyData<QColor>(49, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'lightColor'
        QtMocHelpers::PropertyData<QColor>(50, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'baseGradients'
        QtMocHelpers::PropertyData<QList<QLinearGradient>>(51, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'singleHighlightGradient'
        QtMocHelpers::PropertyData<QLinearGradient>(52, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'multiHighlightGradient'
        QtMocHelpers::PropertyData<QLinearGradient>(53, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'lightStrength'
        QtMocHelpers::PropertyData<float>(54, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'ambientLightStrength'
        QtMocHelpers::PropertyData<float>(55, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'highlightLightStrength'
        QtMocHelpers::PropertyData<float>(56, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'labelBorderEnabled'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'font'
        QtMocHelpers::PropertyData<QFont>(33, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17),
        // property 'backgroundEnabled'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'gridEnabled'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'labelBackgroundEnabled'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'colorStyle'
        QtMocHelpers::PropertyData<enum ColorStyle>(61, 0x80000000 | 62, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 21),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ColorStyle'
        QtMocHelpers::EnumData<enum ColorStyle>(62, 62, QMC::EnumFlags{}).add({
            {   63, ColorStyle::ColorStyleUniform },
            {   64, ColorStyle::ColorStyleObjectGradient },
            {   65, ColorStyle::ColorStyleRangeGradient },
        }),
        // enum 'Theme'
        QtMocHelpers::EnumData<enum Theme>(41, 41, QMC::EnumFlags{}).add({
            {   66, Theme::ThemeQt },
            {   67, Theme::ThemePrimaryColors },
            {   68, Theme::ThemeDigia },
            {   69, Theme::ThemeStoneMoss },
            {   70, Theme::ThemeArmyBlue },
            {   71, Theme::ThemeRetro },
            {   72, Theme::ThemeEbony },
            {   73, Theme::ThemeIsabelle },
            {   74, Theme::ThemeUserDefined },
        }),
    };
    return QtMocHelpers::metaObjectData<Q3DTheme, qt_meta_tag_ZN8Q3DThemeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Q3DTheme::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Q3DThemeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Q3DThemeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Q3DThemeE_t>.metaTypes,
    nullptr
} };

void Q3DTheme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Q3DTheme *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->typeChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme::Theme>>(_a[1]))); break;
        case 1: _t->baseColorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QColor>>>(_a[1]))); break;
        case 2: _t->backgroundColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->windowColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->labelTextColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->labelBackgroundColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->gridLineColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->singleHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 8: _t->multiHighlightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->lightColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 10: _t->baseGradientsChanged((*reinterpret_cast<std::add_pointer_t<QList<QLinearGradient>>>(_a[1]))); break;
        case 11: _t->singleHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 12: _t->multiHighlightGradientChanged((*reinterpret_cast<std::add_pointer_t<QLinearGradient>>(_a[1]))); break;
        case 13: _t->lightStrengthChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 14: _t->ambientLightStrengthChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 15: _t->highlightLightStrengthChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 16: _t->labelBorderEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 18: _t->backgroundEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->gridEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->labelBackgroundEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->colorStyleChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme::ColorStyle>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(Q3DTheme::Theme )>(_a, &Q3DTheme::typeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QList<QColor> & )>(_a, &Q3DTheme::baseColorsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::backgroundColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::windowColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::labelTextColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::labelBackgroundColorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::gridLineColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::singleHighlightColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::multiHighlightColorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QColor & )>(_a, &Q3DTheme::lightColorChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QList<QLinearGradient> & )>(_a, &Q3DTheme::baseGradientsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QLinearGradient & )>(_a, &Q3DTheme::singleHighlightGradientChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QLinearGradient & )>(_a, &Q3DTheme::multiHighlightGradientChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(float )>(_a, &Q3DTheme::lightStrengthChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(float )>(_a, &Q3DTheme::ambientLightStrengthChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(float )>(_a, &Q3DTheme::highlightLightStrengthChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(bool )>(_a, &Q3DTheme::labelBorderEnabledChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(const QFont & )>(_a, &Q3DTheme::fontChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(bool )>(_a, &Q3DTheme::backgroundEnabledChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(bool )>(_a, &Q3DTheme::gridEnabledChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(bool )>(_a, &Q3DTheme::labelBackgroundEnabledChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DTheme::*)(Q3DTheme::ColorStyle )>(_a, &Q3DTheme::colorStyleChanged, 21))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Theme*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QList<QColor>*>(_v) = _t->baseColors(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->backgroundColor(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->windowColor(); break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->labelTextColor(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->labelBackgroundColor(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->gridLineColor(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->singleHighlightColor(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->multiHighlightColor(); break;
        case 9: *reinterpret_cast<QColor*>(_v) = _t->lightColor(); break;
        case 10: *reinterpret_cast<QList<QLinearGradient>*>(_v) = _t->baseGradients(); break;
        case 11: *reinterpret_cast<QLinearGradient*>(_v) = _t->singleHighlightGradient(); break;
        case 12: *reinterpret_cast<QLinearGradient*>(_v) = _t->multiHighlightGradient(); break;
        case 13: *reinterpret_cast<float*>(_v) = _t->lightStrength(); break;
        case 14: *reinterpret_cast<float*>(_v) = _t->ambientLightStrength(); break;
        case 15: *reinterpret_cast<float*>(_v) = _t->highlightLightStrength(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isLabelBorderEnabled(); break;
        case 17: *reinterpret_cast<QFont*>(_v) = _t->font(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isBackgroundEnabled(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isGridEnabled(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->isLabelBackgroundEnabled(); break;
        case 21: *reinterpret_cast<enum ColorStyle*>(_v) = _t->colorStyle(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setType(*reinterpret_cast<enum Theme*>(_v)); break;
        case 1: _t->setBaseColors(*reinterpret_cast<QList<QColor>*>(_v)); break;
        case 2: _t->setBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setWindowColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setLabelTextColor(*reinterpret_cast<QColor*>(_v)); break;
        case 5: _t->setLabelBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->setGridLineColor(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setSingleHighlightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setMultiHighlightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 9: _t->setLightColor(*reinterpret_cast<QColor*>(_v)); break;
        case 10: _t->setBaseGradients(*reinterpret_cast<QList<QLinearGradient>*>(_v)); break;
        case 11: _t->setSingleHighlightGradient(*reinterpret_cast<QLinearGradient*>(_v)); break;
        case 12: _t->setMultiHighlightGradient(*reinterpret_cast<QLinearGradient*>(_v)); break;
        case 13: _t->setLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 14: _t->setAmbientLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 15: _t->setHighlightLightStrength(*reinterpret_cast<float*>(_v)); break;
        case 16: _t->setLabelBorderEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setFont(*reinterpret_cast<QFont*>(_v)); break;
        case 18: _t->setBackgroundEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setGridEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setLabelBackgroundEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setColorStyle(*reinterpret_cast<enum ColorStyle*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Q3DTheme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Q3DTheme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Q3DThemeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Q3DTheme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void Q3DTheme::typeChanged(Q3DTheme::Theme _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Q3DTheme::baseColorsChanged(const QList<QColor> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Q3DTheme::backgroundColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Q3DTheme::windowColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Q3DTheme::labelTextColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Q3DTheme::labelBackgroundColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Q3DTheme::gridLineColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Q3DTheme::singleHighlightColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Q3DTheme::multiHighlightColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Q3DTheme::lightColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Q3DTheme::baseGradientsChanged(const QList<QLinearGradient> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Q3DTheme::singleHighlightGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Q3DTheme::multiHighlightGradientChanged(const QLinearGradient & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Q3DTheme::lightStrengthChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Q3DTheme::ambientLightStrengthChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Q3DTheme::highlightLightStrengthChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Q3DTheme::labelBorderEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Q3DTheme::fontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Q3DTheme::backgroundEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Q3DTheme::gridEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Q3DTheme::labelBackgroundEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Q3DTheme::colorStyleChanged(Q3DTheme::ColorStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
