/****************************************************************************
** Meta object code from reading C++ file 'q3dcamera.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/engine/q3dcamera.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dcamera.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Q3DCameraE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DCamera::qt_create_metaobjectdata<qt_meta_tag_ZN9Q3DCameraE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DCamera",
        "xRotationChanged",
        "",
        "rotation",
        "yRotationChanged",
        "zoomLevelChanged",
        "zoomLevel",
        "cameraPresetChanged",
        "Q3DCamera::CameraPreset",
        "preset",
        "wrapXRotationChanged",
        "isEnabled",
        "wrapYRotationChanged",
        "targetChanged",
        "QVector3D",
        "target",
        "minZoomLevelChanged",
        "maxZoomLevelChanged",
        "xRotation",
        "yRotation",
        "cameraPreset",
        "CameraPreset",
        "wrapXRotation",
        "wrapYRotation",
        "minZoomLevel",
        "maxZoomLevel",
        "CameraPresetNone",
        "CameraPresetFrontLow",
        "CameraPresetFront",
        "CameraPresetFrontHigh",
        "CameraPresetLeftLow",
        "CameraPresetLeft",
        "CameraPresetLeftHigh",
        "CameraPresetRightLow",
        "CameraPresetRight",
        "CameraPresetRightHigh",
        "CameraPresetBehindLow",
        "CameraPresetBehind",
        "CameraPresetBehindHigh",
        "CameraPresetIsometricLeft",
        "CameraPresetIsometricLeftHigh",
        "CameraPresetIsometricRight",
        "CameraPresetIsometricRightHigh",
        "CameraPresetDirectlyAbove",
        "CameraPresetDirectlyAboveCW45",
        "CameraPresetDirectlyAboveCCW45",
        "CameraPresetFrontBelow",
        "CameraPresetLeftBelow",
        "CameraPresetRightBelow",
        "CameraPresetBehindBelow",
        "CameraPresetDirectlyBelow"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'xRotationChanged'
        QtMocHelpers::SignalData<void(float)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 3 },
        }}),
        // Signal 'yRotationChanged'
        QtMocHelpers::SignalData<void(float)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 3 },
        }}),
        // Signal 'zoomLevelChanged'
        QtMocHelpers::SignalData<void(float)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 6 },
        }}),
        // Signal 'cameraPresetChanged'
        QtMocHelpers::SignalData<void(Q3DCamera::CameraPreset)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'wrapXRotationChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'wrapYRotationChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'targetChanged'
        QtMocHelpers::RevisionedSignalData<void(const QVector3D &)>(13, 2, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'minZoomLevelChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(16, 2, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::Float, 6 },
        }}),
        // Signal 'maxZoomLevelChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(17, 2, QMC::AccessPublic, 0x102, QMetaType::Void, {{
            { QMetaType::Float, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'xRotation'
        QtMocHelpers::PropertyData<float>(18, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'yRotation'
        QtMocHelpers::PropertyData<float>(19, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'zoomLevel'
        QtMocHelpers::PropertyData<float>(6, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'cameraPreset'
        QtMocHelpers::PropertyData<enum CameraPreset>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'wrapXRotation'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'wrapYRotation'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'target'
        QtMocHelpers::PropertyData<QVector3D>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6, 0x102),
        // property 'minZoomLevel'
        QtMocHelpers::PropertyData<float>(24, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7, 0x102),
        // property 'maxZoomLevel'
        QtMocHelpers::PropertyData<float>(25, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8, 0x102),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CameraPreset'
        QtMocHelpers::EnumData<enum CameraPreset>(21, 21, QMC::EnumFlags{}).add({
            {   26, CameraPreset::CameraPresetNone },
            {   27, CameraPreset::CameraPresetFrontLow },
            {   28, CameraPreset::CameraPresetFront },
            {   29, CameraPreset::CameraPresetFrontHigh },
            {   30, CameraPreset::CameraPresetLeftLow },
            {   31, CameraPreset::CameraPresetLeft },
            {   32, CameraPreset::CameraPresetLeftHigh },
            {   33, CameraPreset::CameraPresetRightLow },
            {   34, CameraPreset::CameraPresetRight },
            {   35, CameraPreset::CameraPresetRightHigh },
            {   36, CameraPreset::CameraPresetBehindLow },
            {   37, CameraPreset::CameraPresetBehind },
            {   38, CameraPreset::CameraPresetBehindHigh },
            {   39, CameraPreset::CameraPresetIsometricLeft },
            {   40, CameraPreset::CameraPresetIsometricLeftHigh },
            {   41, CameraPreset::CameraPresetIsometricRight },
            {   42, CameraPreset::CameraPresetIsometricRightHigh },
            {   43, CameraPreset::CameraPresetDirectlyAbove },
            {   44, CameraPreset::CameraPresetDirectlyAboveCW45 },
            {   45, CameraPreset::CameraPresetDirectlyAboveCCW45 },
            {   46, CameraPreset::CameraPresetFrontBelow },
            {   47, CameraPreset::CameraPresetLeftBelow },
            {   48, CameraPreset::CameraPresetRightBelow },
            {   49, CameraPreset::CameraPresetBehindBelow },
            {   50, CameraPreset::CameraPresetDirectlyBelow },
        }),
    };
    return QtMocHelpers::metaObjectData<Q3DCamera, qt_meta_tag_ZN9Q3DCameraE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Q3DCamera::staticMetaObject = { {
    QMetaObject::SuperData::link<Q3DObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Q3DCameraE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Q3DCameraE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Q3DCameraE_t>.metaTypes,
    nullptr
} };

void Q3DCamera::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Q3DCamera *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->xRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->yRotationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 2: _t->zoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->cameraPresetChanged((*reinterpret_cast<std::add_pointer_t<Q3DCamera::CameraPreset>>(_a[1]))); break;
        case 4: _t->wrapXRotationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->wrapYRotationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->targetChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 7: _t->minZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 8: _t->maxZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(float )>(_a, &Q3DCamera::xRotationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(float )>(_a, &Q3DCamera::yRotationChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(float )>(_a, &Q3DCamera::zoomLevelChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(Q3DCamera::CameraPreset )>(_a, &Q3DCamera::cameraPresetChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(bool )>(_a, &Q3DCamera::wrapXRotationChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(bool )>(_a, &Q3DCamera::wrapYRotationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(const QVector3D & )>(_a, &Q3DCamera::targetChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(float )>(_a, &Q3DCamera::minZoomLevelChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DCamera::*)(float )>(_a, &Q3DCamera::maxZoomLevelChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<float*>(_v) = _t->xRotation(); break;
        case 1: *reinterpret_cast<float*>(_v) = _t->yRotation(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->zoomLevel(); break;
        case 3: *reinterpret_cast<enum CameraPreset*>(_v) = _t->cameraPreset(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->wrapXRotation(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->wrapYRotation(); break;
        case 6: *reinterpret_cast<QVector3D*>(_v) = _t->target(); break;
        case 7: *reinterpret_cast<float*>(_v) = _t->minZoomLevel(); break;
        case 8: *reinterpret_cast<float*>(_v) = _t->maxZoomLevel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setXRotation(*reinterpret_cast<float*>(_v)); break;
        case 1: _t->setYRotation(*reinterpret_cast<float*>(_v)); break;
        case 2: _t->setZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->setCameraPreset(*reinterpret_cast<enum CameraPreset*>(_v)); break;
        case 4: _t->setWrapXRotation(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setWrapYRotation(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setTarget(*reinterpret_cast<QVector3D*>(_v)); break;
        case 7: _t->setMinZoomLevel(*reinterpret_cast<float*>(_v)); break;
        case 8: _t->setMaxZoomLevel(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Q3DCamera::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Q3DCamera::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Q3DCameraE_t>.strings))
        return static_cast<void*>(this);
    return Q3DObject::qt_metacast(_clname);
}

int Q3DCamera::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Q3DObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Q3DCamera::xRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Q3DCamera::yRotationChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Q3DCamera::zoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Q3DCamera::cameraPresetChanged(Q3DCamera::CameraPreset _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Q3DCamera::wrapXRotationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Q3DCamera::wrapYRotationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Q3DCamera::targetChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Q3DCamera::minZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Q3DCamera::maxZoomLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
