/****************************************************************************
** Meta object code from reading C++ file 'qabstract3dgraph.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/engine/qabstract3dgraph.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstract3dgraph.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QAbstract3DGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DGraph::qt_create_metaobjectdata<qt_meta_tag_ZN16QAbstract3DGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DGraph",
        "activeInputHandlerChanged",
        "",
        "QAbstract3DInputHandler*",
        "inputHandler",
        "activeThemeChanged",
        "Q3DTheme*",
        "theme",
        "selectionModeChanged",
        "QAbstract3DGraph::SelectionFlags",
        "mode",
        "shadowQualityChanged",
        "QAbstract3DGraph::ShadowQuality",
        "quality",
        "selectedElementChanged",
        "QAbstract3DGraph::ElementType",
        "type",
        "measureFpsChanged",
        "enabled",
        "currentFpsChanged",
        "fps",
        "orthoProjectionChanged",
        "aspectRatioChanged",
        "ratio",
        "optimizationHintsChanged",
        "QAbstract3DGraph::OptimizationHints",
        "hints",
        "polarChanged",
        "radialLabelOffsetChanged",
        "offset",
        "horizontalAspectRatioChanged",
        "reflectionChanged",
        "reflectivityChanged",
        "reflectivity",
        "localeChanged",
        "locale",
        "queriedGraphPositionChanged",
        "QVector3D",
        "data",
        "marginChanged",
        "margin",
        "activeInputHandler",
        "activeTheme",
        "selectionMode",
        "SelectionFlags",
        "shadowQuality",
        "ShadowQuality",
        "scene",
        "Q3DScene*",
        "measureFps",
        "currentFps",
        "orthoProjection",
        "selectedElement",
        "ElementType",
        "aspectRatio",
        "optimizationHints",
        "OptimizationHints",
        "polar",
        "radialLabelOffset",
        "horizontalAspectRatio",
        "reflection",
        "queriedGraphPosition",
        "SelectionFlag",
        "SelectionNone",
        "SelectionItem",
        "SelectionRow",
        "SelectionItemAndRow",
        "SelectionColumn",
        "SelectionItemAndColumn",
        "SelectionRowAndColumn",
        "SelectionItemRowAndColumn",
        "SelectionSlice",
        "SelectionMultiSeries",
        "ShadowQualityNone",
        "ShadowQualityLow",
        "ShadowQualityMedium",
        "ShadowQualityHigh",
        "ShadowQualitySoftLow",
        "ShadowQualitySoftMedium",
        "ShadowQualitySoftHigh",
        "ElementNone",
        "ElementSeries",
        "ElementAxisXLabel",
        "ElementAxisYLabel",
        "ElementAxisZLabel",
        "ElementCustomItem",
        "OptimizationHint",
        "OptimizationDefault",
        "OptimizationStatic"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activeInputHandlerChanged'
        QtMocHelpers::SignalData<void(QAbstract3DInputHandler *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'activeThemeChanged'
        QtMocHelpers::SignalData<void(Q3DTheme *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'selectionModeChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::SelectionFlags)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'shadowQualityChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::ShadowQuality)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'selectedElementChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::ElementType)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'measureFpsChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'currentFpsChanged'
        QtMocHelpers::SignalData<void(qreal)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 20 },
        }}),
        // Signal 'orthoProjectionChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'aspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 23 },
        }}),
        // Signal 'optimizationHintsChanged'
        QtMocHelpers::SignalData<void(QAbstract3DGraph::OptimizationHints)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'polarChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'radialLabelOffsetChanged'
        QtMocHelpers::SignalData<void(float)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 29 },
        }}),
        // Signal 'horizontalAspectRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 23 },
        }}),
        // Signal 'reflectionChanged'
        QtMocHelpers::SignalData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'reflectivityChanged'
        QtMocHelpers::SignalData<void(qreal)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 33 },
        }}),
        // Signal 'localeChanged'
        QtMocHelpers::SignalData<void(const QLocale &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QLocale, 35 },
        }}),
        // Signal 'queriedGraphPositionChanged'
        QtMocHelpers::SignalData<void(const QVector3D &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Signal 'marginChanged'
        QtMocHelpers::SignalData<void(qreal)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'activeInputHandler'
        QtMocHelpers::PropertyData<QAbstract3DInputHandler*>(41, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'activeTheme'
        QtMocHelpers::PropertyData<Q3DTheme*>(42, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'selectionMode'
        QtMocHelpers::PropertyData<SelectionFlags>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'shadowQuality'
        QtMocHelpers::PropertyData<enum ShadowQuality>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'scene'
        QtMocHelpers::PropertyData<Q3DScene*>(47, 0x80000000 | 48, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'measureFps'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'currentFps'
        QtMocHelpers::PropertyData<qreal>(50, QMetaType::QReal, QMC::DefaultPropertyFlags, 6),
        // property 'orthoProjection'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'selectedElement'
        QtMocHelpers::PropertyData<enum ElementType>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'aspectRatio'
        QtMocHelpers::PropertyData<qreal>(54, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'optimizationHints'
        QtMocHelpers::PropertyData<OptimizationHints>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'polar'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'radialLabelOffset'
        QtMocHelpers::PropertyData<float>(58, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'horizontalAspectRatio'
        QtMocHelpers::PropertyData<qreal>(59, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'reflection'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'reflectivity'
        QtMocHelpers::PropertyData<qreal>(33, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'locale'
        QtMocHelpers::PropertyData<QLocale>(35, QMetaType::QLocale, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'queriedGraphPosition'
        QtMocHelpers::PropertyData<QVector3D>(61, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 16),
        // property 'margin'
        QtMocHelpers::PropertyData<qreal>(40, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelectionFlag'
        QtMocHelpers::EnumData<enum SelectionFlag>(62, 62, QMC::EnumFlags{}).add({
            {   63, SelectionFlag::SelectionNone },
            {   64, SelectionFlag::SelectionItem },
            {   65, SelectionFlag::SelectionRow },
            {   66, SelectionFlag::SelectionItemAndRow },
            {   67, SelectionFlag::SelectionColumn },
            {   68, SelectionFlag::SelectionItemAndColumn },
            {   69, SelectionFlag::SelectionRowAndColumn },
            {   70, SelectionFlag::SelectionItemRowAndColumn },
            {   71, SelectionFlag::SelectionSlice },
            {   72, SelectionFlag::SelectionMultiSeries },
        }),
        // flag 'SelectionFlags'
        QtMocHelpers::EnumData<SelectionFlags>(44, 62, QMC::EnumIsFlag).add({
            {   63, SelectionFlag::SelectionNone },
            {   64, SelectionFlag::SelectionItem },
            {   65, SelectionFlag::SelectionRow },
            {   66, SelectionFlag::SelectionItemAndRow },
            {   67, SelectionFlag::SelectionColumn },
            {   68, SelectionFlag::SelectionItemAndColumn },
            {   69, SelectionFlag::SelectionRowAndColumn },
            {   70, SelectionFlag::SelectionItemRowAndColumn },
            {   71, SelectionFlag::SelectionSlice },
            {   72, SelectionFlag::SelectionMultiSeries },
        }),
        // enum 'ShadowQuality'
        QtMocHelpers::EnumData<enum ShadowQuality>(46, 46, QMC::EnumFlags{}).add({
            {   73, ShadowQuality::ShadowQualityNone },
            {   74, ShadowQuality::ShadowQualityLow },
            {   75, ShadowQuality::ShadowQualityMedium },
            {   76, ShadowQuality::ShadowQualityHigh },
            {   77, ShadowQuality::ShadowQualitySoftLow },
            {   78, ShadowQuality::ShadowQualitySoftMedium },
            {   79, ShadowQuality::ShadowQualitySoftHigh },
        }),
        // enum 'ElementType'
        QtMocHelpers::EnumData<enum ElementType>(53, 53, QMC::EnumFlags{}).add({
            {   80, ElementType::ElementNone },
            {   81, ElementType::ElementSeries },
            {   82, ElementType::ElementAxisXLabel },
            {   83, ElementType::ElementAxisYLabel },
            {   84, ElementType::ElementAxisZLabel },
            {   85, ElementType::ElementCustomItem },
        }),
        // enum 'OptimizationHint'
        QtMocHelpers::EnumData<enum OptimizationHint>(86, 86, QMC::EnumFlags{}).add({
            {   87, OptimizationHint::OptimizationDefault },
            {   88, OptimizationHint::OptimizationStatic },
        }),
        // flag 'OptimizationHints'
        QtMocHelpers::EnumData<OptimizationHints>(56, 86, QMC::EnumIsFlag).add({
            {   87, OptimizationHint::OptimizationDefault },
            {   88, OptimizationHint::OptimizationStatic },
        }),
    };
    return QtMocHelpers::metaObjectData<QAbstract3DGraph, qt_meta_tag_ZN16QAbstract3DGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstract3DGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<QWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QAbstract3DGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QAbstract3DGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QAbstract3DGraphE_t>.metaTypes,
    nullptr
} };

void QAbstract3DGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstract3DGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activeInputHandlerChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DInputHandler*>>(_a[1]))); break;
        case 1: _t->activeThemeChanged((*reinterpret_cast<std::add_pointer_t<Q3DTheme*>>(_a[1]))); break;
        case 2: _t->selectionModeChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::SelectionFlags>>(_a[1]))); break;
        case 3: _t->shadowQualityChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::ShadowQuality>>(_a[1]))); break;
        case 4: _t->selectedElementChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::ElementType>>(_a[1]))); break;
        case 5: _t->measureFpsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->currentFpsChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->orthoProjectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->aspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 9: _t->optimizationHintsChanged((*reinterpret_cast<std::add_pointer_t<QAbstract3DGraph::OptimizationHints>>(_a[1]))); break;
        case 10: _t->polarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->radialLabelOffsetChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 12: _t->horizontalAspectRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 13: _t->reflectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->reflectivityChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 15: _t->localeChanged((*reinterpret_cast<std::add_pointer_t<QLocale>>(_a[1]))); break;
        case 16: _t->queriedGraphPositionChanged((*reinterpret_cast<std::add_pointer_t<QVector3D>>(_a[1]))); break;
        case 17: _t->marginChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstract3DInputHandler* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Q3DTheme* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(QAbstract3DInputHandler * )>(_a, &QAbstract3DGraph::activeInputHandlerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(Q3DTheme * )>(_a, &QAbstract3DGraph::activeThemeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(QAbstract3DGraph::SelectionFlags )>(_a, &QAbstract3DGraph::selectionModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(QAbstract3DGraph::ShadowQuality )>(_a, &QAbstract3DGraph::shadowQualityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(QAbstract3DGraph::ElementType )>(_a, &QAbstract3DGraph::selectedElementChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(bool )>(_a, &QAbstract3DGraph::measureFpsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(qreal )>(_a, &QAbstract3DGraph::currentFpsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(bool )>(_a, &QAbstract3DGraph::orthoProjectionChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(qreal )>(_a, &QAbstract3DGraph::aspectRatioChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(QAbstract3DGraph::OptimizationHints )>(_a, &QAbstract3DGraph::optimizationHintsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(bool )>(_a, &QAbstract3DGraph::polarChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(float )>(_a, &QAbstract3DGraph::radialLabelOffsetChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(qreal )>(_a, &QAbstract3DGraph::horizontalAspectRatioChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(bool )>(_a, &QAbstract3DGraph::reflectionChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(qreal )>(_a, &QAbstract3DGraph::reflectivityChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(const QLocale & )>(_a, &QAbstract3DGraph::localeChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(const QVector3D & )>(_a, &QAbstract3DGraph::queriedGraphPositionChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstract3DGraph::*)(qreal )>(_a, &QAbstract3DGraph::marginChanged, 17))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DScene* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Q3DTheme* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstract3DInputHandler* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstract3DInputHandler**>(_v) = _t->activeInputHandler(); break;
        case 1: *reinterpret_cast<Q3DTheme**>(_v) = _t->activeTheme(); break;
        case 2: QtMocHelpers::assignFlags<SelectionFlags>(_v, _t->selectionMode()); break;
        case 3: *reinterpret_cast<enum ShadowQuality*>(_v) = _t->shadowQuality(); break;
        case 4: *reinterpret_cast<Q3DScene**>(_v) = _t->scene(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->measureFps(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->currentFps(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isOrthoProjection(); break;
        case 8: *reinterpret_cast<enum ElementType*>(_v) = _t->selectedElement(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->aspectRatio(); break;
        case 10: QtMocHelpers::assignFlags<OptimizationHints>(_v, _t->optimizationHints()); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isPolar(); break;
        case 12: *reinterpret_cast<float*>(_v) = _t->radialLabelOffset(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->horizontalAspectRatio(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isReflection(); break;
        case 15: *reinterpret_cast<qreal*>(_v) = _t->reflectivity(); break;
        case 16: *reinterpret_cast<QLocale*>(_v) = _t->locale(); break;
        case 17: *reinterpret_cast<QVector3D*>(_v) = _t->queriedGraphPosition(); break;
        case 18: *reinterpret_cast<qreal*>(_v) = _t->margin(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActiveInputHandler(*reinterpret_cast<QAbstract3DInputHandler**>(_v)); break;
        case 1: _t->setActiveTheme(*reinterpret_cast<Q3DTheme**>(_v)); break;
        case 2: _t->setSelectionMode(*reinterpret_cast<SelectionFlags*>(_v)); break;
        case 3: _t->setShadowQuality(*reinterpret_cast<enum ShadowQuality*>(_v)); break;
        case 5: _t->setMeasureFps(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setOrthoProjection(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setOptimizationHints(*reinterpret_cast<OptimizationHints*>(_v)); break;
        case 11: _t->setPolar(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setRadialLabelOffset(*reinterpret_cast<float*>(_v)); break;
        case 13: _t->setHorizontalAspectRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 14: _t->setReflection(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setReflectivity(*reinterpret_cast<qreal*>(_v)); break;
        case 16: _t->setLocale(*reinterpret_cast<QLocale*>(_v)); break;
        case 18: _t->setMargin(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstract3DGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstract3DGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QAbstract3DGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QOpenGLFunctions"))
        return static_cast< QOpenGLFunctions*>(this);
    return QWindow::qt_metacast(_clname);
}

int QAbstract3DGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void QAbstract3DGraph::activeInputHandlerChanged(QAbstract3DInputHandler * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstract3DGraph::activeThemeChanged(Q3DTheme * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstract3DGraph::selectionModeChanged(QAbstract3DGraph::SelectionFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstract3DGraph::shadowQualityChanged(QAbstract3DGraph::ShadowQuality _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstract3DGraph::selectedElementChanged(QAbstract3DGraph::ElementType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAbstract3DGraph::measureFpsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QAbstract3DGraph::currentFpsChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstract3DGraph::orthoProjectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstract3DGraph::aspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstract3DGraph::optimizationHintsChanged(QAbstract3DGraph::OptimizationHints _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstract3DGraph::polarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstract3DGraph::radialLabelOffsetChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstract3DGraph::horizontalAspectRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstract3DGraph::reflectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QAbstract3DGraph::reflectivityChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QAbstract3DGraph::localeChanged(const QLocale & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QAbstract3DGraph::queriedGraphPositionChanged(const QVector3D & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QAbstract3DGraph::marginChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}
QT_WARNING_POP
