/****************************************************************************
** Meta object code from reading C++ file 'q3dbars.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/engine/q3dbars.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtDataVisualization/qbar3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'q3dbars.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Q3DBarsE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DBars::qt_create_metaobjectdata<qt_meta_tag_ZN7Q3DBarsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DBars",
        "multiSeriesUniformChanged",
        "",
        "uniform",
        "barThicknessChanged",
        "thicknessRatio",
        "barSpacingChanged",
        "QSizeF",
        "spacing",
        "barSpacingRelativeChanged",
        "relative",
        "barSeriesMarginChanged",
        "margin",
        "rowAxisChanged",
        "QCategory3DAxis*",
        "axis",
        "columnAxisChanged",
        "valueAxisChanged",
        "QValue3DAxis*",
        "primarySeriesChanged",
        "QBar3DSeries*",
        "series",
        "selectedSeriesChanged",
        "floorLevelChanged",
        "level",
        "multiSeriesUniform",
        "barThickness",
        "barSpacing",
        "barSpacingRelative",
        "barSeriesMargin",
        "rowAxis",
        "columnAxis",
        "valueAxis",
        "primarySeries",
        "selectedSeries",
        "floorLevel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'multiSeriesUniformChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'barThicknessChanged'
        QtMocHelpers::SignalData<void(float)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 5 },
        }}),
        // Signal 'barSpacingChanged'
        QtMocHelpers::SignalData<void(const QSizeF &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'barSpacingRelativeChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'barSeriesMarginChanged'
        QtMocHelpers::RevisionedSignalData<void(const QSizeF &)>(11, 2, QMC::AccessPublic, 0x603, QMetaType::Void, {{
            { 0x80000000 | 7, 12 },
        }}),
        // Signal 'rowAxisChanged'
        QtMocHelpers::SignalData<void(QCategory3DAxis *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'columnAxisChanged'
        QtMocHelpers::SignalData<void(QCategory3DAxis *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'valueAxisChanged'
        QtMocHelpers::SignalData<void(QValue3DAxis *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 15 },
        }}),
        // Signal 'primarySeriesChanged'
        QtMocHelpers::SignalData<void(QBar3DSeries *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'selectedSeriesChanged'
        QtMocHelpers::SignalData<void(QBar3DSeries *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'floorLevelChanged'
        QtMocHelpers::SignalData<void(float)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'multiSeriesUniform'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'barThickness'
        QtMocHelpers::PropertyData<float>(26, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'barSpacing'
        QtMocHelpers::PropertyData<QSizeF>(27, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'barSpacingRelative'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'barSeriesMargin'
        QtMocHelpers::PropertyData<QSizeF>(29, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4, 0x603),
        // property 'rowAxis'
        QtMocHelpers::PropertyData<QCategory3DAxis*>(30, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'columnAxis'
        QtMocHelpers::PropertyData<QCategory3DAxis*>(31, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'valueAxis'
        QtMocHelpers::PropertyData<QValue3DAxis*>(32, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'primarySeries'
        QtMocHelpers::PropertyData<QBar3DSeries*>(33, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'selectedSeries'
        QtMocHelpers::PropertyData<QBar3DSeries*>(34, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
        // property 'floorLevel'
        QtMocHelpers::PropertyData<float>(35, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Q3DBars, qt_meta_tag_ZN7Q3DBarsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Q3DBars::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstract3DGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Q3DBarsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Q3DBarsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Q3DBarsE_t>.metaTypes,
    nullptr
} };

void Q3DBars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Q3DBars *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->multiSeriesUniformChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->barThicknessChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 2: _t->barSpacingChanged((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        case 3: _t->barSpacingRelativeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->barSeriesMarginChanged((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        case 5: _t->rowAxisChanged((*reinterpret_cast<std::add_pointer_t<QCategory3DAxis*>>(_a[1]))); break;
        case 6: _t->columnAxisChanged((*reinterpret_cast<std::add_pointer_t<QCategory3DAxis*>>(_a[1]))); break;
        case 7: _t->valueAxisChanged((*reinterpret_cast<std::add_pointer_t<QValue3DAxis*>>(_a[1]))); break;
        case 8: _t->primarySeriesChanged((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        case 9: _t->selectedSeriesChanged((*reinterpret_cast<std::add_pointer_t<QBar3DSeries*>>(_a[1]))); break;
        case 10: _t->floorLevelChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCategory3DAxis* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QCategory3DAxis* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QValue3DAxis* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBar3DSeries* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(bool )>(_a, &Q3DBars::multiSeriesUniformChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(float )>(_a, &Q3DBars::barThicknessChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(const QSizeF & )>(_a, &Q3DBars::barSpacingChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(bool )>(_a, &Q3DBars::barSpacingRelativeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(const QSizeF & )>(_a, &Q3DBars::barSeriesMarginChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(QCategory3DAxis * )>(_a, &Q3DBars::rowAxisChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(QCategory3DAxis * )>(_a, &Q3DBars::columnAxisChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(QValue3DAxis * )>(_a, &Q3DBars::valueAxisChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(QBar3DSeries * )>(_a, &Q3DBars::primarySeriesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(QBar3DSeries * )>(_a, &Q3DBars::selectedSeriesChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Q3DBars::*)(float )>(_a, &Q3DBars::floorLevelChanged, 10))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBar3DSeries* >(); break;
        case 6:
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCategory3DAxis* >(); break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValue3DAxis* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isMultiSeriesUniform(); break;
        case 1: *reinterpret_cast<float*>(_v) = _t->barThickness(); break;
        case 2: *reinterpret_cast<QSizeF*>(_v) = _t->barSpacing(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isBarSpacingRelative(); break;
        case 4: *reinterpret_cast<QSizeF*>(_v) = _t->barSeriesMargin(); break;
        case 5: *reinterpret_cast<QCategory3DAxis**>(_v) = _t->rowAxis(); break;
        case 6: *reinterpret_cast<QCategory3DAxis**>(_v) = _t->columnAxis(); break;
        case 7: *reinterpret_cast<QValue3DAxis**>(_v) = _t->valueAxis(); break;
        case 8: *reinterpret_cast<QBar3DSeries**>(_v) = _t->primarySeries(); break;
        case 9: *reinterpret_cast<QBar3DSeries**>(_v) = _t->selectedSeries(); break;
        case 10: *reinterpret_cast<float*>(_v) = _t->floorLevel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMultiSeriesUniform(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setBarThickness(*reinterpret_cast<float*>(_v)); break;
        case 2: _t->setBarSpacing(*reinterpret_cast<QSizeF*>(_v)); break;
        case 3: _t->setBarSpacingRelative(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setBarSeriesMargin(*reinterpret_cast<QSizeF*>(_v)); break;
        case 5: _t->setRowAxis(*reinterpret_cast<QCategory3DAxis**>(_v)); break;
        case 6: _t->setColumnAxis(*reinterpret_cast<QCategory3DAxis**>(_v)); break;
        case 7: _t->setValueAxis(*reinterpret_cast<QValue3DAxis**>(_v)); break;
        case 8: _t->setPrimarySeries(*reinterpret_cast<QBar3DSeries**>(_v)); break;
        case 10: _t->setFloorLevel(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Q3DBars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Q3DBars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Q3DBarsE_t>.strings))
        return static_cast<void*>(this);
    return QAbstract3DGraph::qt_metacast(_clname);
}

int Q3DBars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstract3DGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Q3DBars::multiSeriesUniformChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Q3DBars::barThicknessChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Q3DBars::barSpacingChanged(const QSizeF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Q3DBars::barSpacingRelativeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Q3DBars::barSeriesMarginChanged(const QSizeF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Q3DBars::rowAxisChanged(QCategory3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Q3DBars::columnAxisChanged(QCategory3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Q3DBars::valueAxisChanged(QValue3DAxis * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Q3DBars::primarySeriesChanged(QBar3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Q3DBars::selectedSeriesChanged(QBar3DSeries * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Q3DBars::floorLevelChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
