/****************************************************************************
** Meta object code from reading C++ file 'qheightmapsurfacedataproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualization/data/qheightmapsurfacedataproxy.h"
#include <QtDataVisualization/qsurface3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qheightmapsurfacedataproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto QHeightMapSurfaceDataProxy::qt_create_metaobjectdata<qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHeightMapSurfaceDataProxy",
        "heightMapChanged",
        "",
        "QImage",
        "image",
        "heightMapFileChanged",
        "filename",
        "minXValueChanged",
        "value",
        "maxXValueChanged",
        "minZValueChanged",
        "maxZValueChanged",
        "minYValueChanged",
        "maxYValueChanged",
        "autoScaleYChanged",
        "enabled",
        "heightMap",
        "heightMapFile",
        "minXValue",
        "maxXValue",
        "minZValue",
        "maxZValue",
        "minYValue",
        "maxYValue",
        "autoScaleY"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'heightMapChanged'
        QtMocHelpers::SignalData<void(const QImage &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'heightMapFileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'minXValueChanged'
        QtMocHelpers::SignalData<void(float)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'maxXValueChanged'
        QtMocHelpers::SignalData<void(float)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'minZValueChanged'
        QtMocHelpers::SignalData<void(float)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'maxZValueChanged'
        QtMocHelpers::SignalData<void(float)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'minYValueChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(12, 2, QMC::AccessPublic, 0x603, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'maxYValueChanged'
        QtMocHelpers::RevisionedSignalData<void(float)>(13, 2, QMC::AccessPublic, 0x603, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Signal 'autoScaleYChanged'
        QtMocHelpers::RevisionedSignalData<void(bool)>(14, 2, QMC::AccessPublic, 0x603, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'heightMap'
        QtMocHelpers::PropertyData<QImage>(16, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'heightMapFile'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'minXValue'
        QtMocHelpers::PropertyData<float>(18, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'maxXValue'
        QtMocHelpers::PropertyData<float>(19, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'minZValue'
        QtMocHelpers::PropertyData<float>(20, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'maxZValue'
        QtMocHelpers::PropertyData<float>(21, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'minYValue'
        QtMocHelpers::PropertyData<float>(22, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6, 0x603),
        // property 'maxYValue'
        QtMocHelpers::PropertyData<float>(23, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7, 0x603),
        // property 'autoScaleY'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8, 0x603),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QHeightMapSurfaceDataProxy, qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHeightMapSurfaceDataProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QSurfaceDataProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t>.metaTypes,
    nullptr
} };

void QHeightMapSurfaceDataProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHeightMapSurfaceDataProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->heightMapChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 1: _t->heightMapFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->minXValueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 3: _t->maxXValueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 4: _t->minZValueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 5: _t->maxZValueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 6: _t->minYValueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 7: _t->maxYValueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 8: _t->autoScaleYChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(const QImage & )>(_a, &QHeightMapSurfaceDataProxy::heightMapChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(const QString & )>(_a, &QHeightMapSurfaceDataProxy::heightMapFileChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(float )>(_a, &QHeightMapSurfaceDataProxy::minXValueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(float )>(_a, &QHeightMapSurfaceDataProxy::maxXValueChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(float )>(_a, &QHeightMapSurfaceDataProxy::minZValueChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(float )>(_a, &QHeightMapSurfaceDataProxy::maxZValueChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(float )>(_a, &QHeightMapSurfaceDataProxy::minYValueChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(float )>(_a, &QHeightMapSurfaceDataProxy::maxYValueChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHeightMapSurfaceDataProxy::*)(bool )>(_a, &QHeightMapSurfaceDataProxy::autoScaleYChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QImage*>(_v) = _t->heightMap(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->heightMapFile(); break;
        case 2: *reinterpret_cast<float*>(_v) = _t->minXValue(); break;
        case 3: *reinterpret_cast<float*>(_v) = _t->maxXValue(); break;
        case 4: *reinterpret_cast<float*>(_v) = _t->minZValue(); break;
        case 5: *reinterpret_cast<float*>(_v) = _t->maxZValue(); break;
        case 6: *reinterpret_cast<float*>(_v) = _t->minYValue(); break;
        case 7: *reinterpret_cast<float*>(_v) = _t->maxYValue(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->autoScaleY(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHeightMap(*reinterpret_cast<QImage*>(_v)); break;
        case 1: _t->setHeightMapFile(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setMinXValue(*reinterpret_cast<float*>(_v)); break;
        case 3: _t->setMaxXValue(*reinterpret_cast<float*>(_v)); break;
        case 4: _t->setMinZValue(*reinterpret_cast<float*>(_v)); break;
        case 5: _t->setMaxZValue(*reinterpret_cast<float*>(_v)); break;
        case 6: _t->setMinYValue(*reinterpret_cast<float*>(_v)); break;
        case 7: _t->setMaxYValue(*reinterpret_cast<float*>(_v)); break;
        case 8: _t->setAutoScaleY(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QHeightMapSurfaceDataProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHeightMapSurfaceDataProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QHeightMapSurfaceDataProxyE_t>.strings))
        return static_cast<void*>(this);
    return QSurfaceDataProxy::qt_metacast(_clname);
}

int QHeightMapSurfaceDataProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSurfaceDataProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QHeightMapSurfaceDataProxy::heightMapChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QHeightMapSurfaceDataProxy::heightMapFileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QHeightMapSurfaceDataProxy::minXValueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QHeightMapSurfaceDataProxy::maxXValueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QHeightMapSurfaceDataProxy::minZValueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QHeightMapSurfaceDataProxy::maxZValueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QHeightMapSurfaceDataProxy::minYValueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QHeightMapSurfaceDataProxy::maxYValueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QHeightMapSurfaceDataProxy::autoScaleYChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
