/****************************************************************************
** Meta object code from reading C++ file 'foreigntypes_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtdatavis3d/src/datavisualizationqml/foreigntypes_p.h"
#include <QtDataVisualization/q3dscene.h>
#include <QtDataVisualization/qbar3dseries.h>
#include <QtDataVisualization/qsurface3dseries.h>
#include <QtDataVisualization/qscatter3dseries.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'foreigntypes_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15Q3DSceneForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DSceneForeign::qt_create_metaobjectdata<qt_meta_tag_ZN15Q3DSceneForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DSceneForeign",
        "QML.Element",
        "anonymous",
        "QML.AddedInVersion",
        "256",
        "QML.Foreign",
        "Q3DScene"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Q3DSceneForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DSceneForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Q3DSceneForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Q3DSceneForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15Q3DSceneForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN23Q3DCameraDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DCameraDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN23Q3DCameraDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DCameraDataVisForeign",
        "QML.Element",
        "Camera3D",
        "QML.Foreign",
        "Q3DCamera",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Q3DCameraDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DCameraDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23Q3DCameraDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23Q3DCameraDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23Q3DCameraDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN22Q3DLightDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DLightDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN22Q3DLightDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DLightDataVisForeign",
        "QML.Element",
        "Light3D",
        "QML.Foreign",
        "Q3DLight",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Q3DLightDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DLightDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22Q3DLightDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22Q3DLightDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22Q3DLightDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN29QCategory3DAxisDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QCategory3DAxisDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN29QCategory3DAxisDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCategory3DAxisDataVisForeign",
        "QML.Element",
        "CategoryAxis3D",
        "QML.Foreign",
        "QCategory3DAxis",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QCategory3DAxisDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCategory3DAxisDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QCategory3DAxisDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QCategory3DAxisDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QCategory3DAxisDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN40QHeightMapSurfaceDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QHeightMapSurfaceDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN40QHeightMapSurfaceDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHeightMapSurfaceDataProxyDataVisForeign",
        "QML.Element",
        "HeightMapSurfaceDataProxy",
        "QML.Foreign",
        "QHeightMapSurfaceDataProxy",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QHeightMapSurfaceDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QHeightMapSurfaceDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40QHeightMapSurfaceDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40QHeightMapSurfaceDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN40QHeightMapSurfaceDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN36QItemModelBarDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QItemModelBarDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN36QItemModelBarDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QItemModelBarDataProxyDataVisForeign",
        "QML.Element",
        "ItemModelBarDataProxy",
        "QML.Foreign",
        "QItemModelBarDataProxy",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QItemModelBarDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QItemModelBarDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36QItemModelBarDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36QItemModelBarDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36QItemModelBarDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN40QItemModelScatterDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QItemModelScatterDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN40QItemModelScatterDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QItemModelScatterDataProxyDataVisForeign",
        "QML.Element",
        "ItemModelScatterDataProxy",
        "QML.Foreign",
        "QItemModelScatterDataProxy",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QItemModelScatterDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QItemModelScatterDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40QItemModelScatterDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40QItemModelScatterDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN40QItemModelScatterDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN40QItemModelSurfaceDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QItemModelSurfaceDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN40QItemModelSurfaceDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QItemModelSurfaceDataProxyDataVisForeign",
        "QML.Element",
        "ItemModelSurfaceDataProxy",
        "QML.Foreign",
        "QItemModelSurfaceDataProxy",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QItemModelSurfaceDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QItemModelSurfaceDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40QItemModelSurfaceDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40QItemModelSurfaceDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN40QItemModelSurfaceDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN26QValue3DAxisDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QValue3DAxisDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN26QValue3DAxisDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QValue3DAxisDataVisForeign",
        "QML.Element",
        "ValueAxis3D",
        "QML.Foreign",
        "QValue3DAxis",
        "QML.AddedInVersion",
        "256"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QValue3DAxisDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QValue3DAxisDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QValue3DAxisDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QValue3DAxisDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QValue3DAxisDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN27QCustom3DItemDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QCustom3DItemDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN27QCustom3DItemDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCustom3DItemDataVisForeign",
        "QML.Element",
        "Custom3DItem",
        "QML.Foreign",
        "QCustom3DItem",
        "QML.AddedInVersion",
        "257"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QCustom3DItemDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCustom3DItemDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QCustom3DItemDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QCustom3DItemDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QCustom3DItemDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN28QCustom3DLabelDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QCustom3DLabelDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN28QCustom3DLabelDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCustom3DLabelDataVisForeign",
        "QML.Element",
        "Custom3DLabel",
        "QML.Foreign",
        "QCustom3DLabel",
        "QML.AddedInVersion",
        "257"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QCustom3DLabelDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCustom3DLabelDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QCustom3DLabelDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QCustom3DLabelDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QCustom3DLabelDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN38QLogValue3DAxisFormatterDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QLogValue3DAxisFormatterDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN38QLogValue3DAxisFormatterDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLogValue3DAxisFormatterDataVisForeign",
        "QML.Element",
        "LogValueAxis3DFormatter",
        "QML.Foreign",
        "QLogValue3DAxisFormatter",
        "QML.AddedInVersion",
        "257"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QLogValue3DAxisFormatterDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QLogValue3DAxisFormatterDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38QLogValue3DAxisFormatterDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38QLogValue3DAxisFormatterDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN38QLogValue3DAxisFormatterDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN35QValue3DAxisFormatterDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QValue3DAxisFormatterDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN35QValue3DAxisFormatterDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QValue3DAxisFormatterDataVisForeign",
        "QML.Element",
        "ValueAxis3DFormatter",
        "QML.Foreign",
        "QValue3DAxisFormatter",
        "QML.AddedInVersion",
        "257"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QValue3DAxisFormatterDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QValue3DAxisFormatterDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QValue3DAxisFormatterDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QValue3DAxisFormatterDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QValue3DAxisFormatterDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN29Q3DInputHandlerDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DInputHandlerDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN29Q3DInputHandlerDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DInputHandlerDataVisForeign",
        "QML.Element",
        "InputHandler3D",
        "QML.Foreign",
        "Q3DInputHandler",
        "QML.AddedInVersion",
        "258"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Q3DInputHandlerDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DInputHandlerDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29Q3DInputHandlerDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29Q3DInputHandlerDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29Q3DInputHandlerDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN29QCustom3DVolumeDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QCustom3DVolumeDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN29QCustom3DVolumeDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QCustom3DVolumeDataVisForeign",
        "QML.Element",
        "Custom3DVolume",
        "QML.Foreign",
        "QCustom3DVolume",
        "QML.AddedInVersion",
        "258"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QCustom3DVolumeDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QCustom3DVolumeDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QCustom3DVolumeDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QCustom3DVolumeDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QCustom3DVolumeDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN34QTouch3DInputHandlerDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QTouch3DInputHandlerDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN34QTouch3DInputHandlerDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QTouch3DInputHandlerDataVisForeign",
        "QML.Element",
        "TouchInputHandler3D",
        "QML.Foreign",
        "QTouch3DInputHandler",
        "QML.AddedInVersion",
        "258"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QTouch3DInputHandlerDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QTouch3DInputHandlerDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34QTouch3DInputHandlerDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34QTouch3DInputHandlerDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34QTouch3DInputHandlerDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN22Q3DThemeDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DThemeDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN22Q3DThemeDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DThemeDataVisForeign",
        "QML.Element",
        "Q3DTheme",
        "QML.Foreign",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: Q3DTheme, use Theme3D instead."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
            {    6,    7 },
            {    8,    9 },
    });
    return QtMocHelpers::metaObjectData<Q3DThemeDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DThemeDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22Q3DThemeDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22Q3DThemeDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22Q3DThemeDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN26QBar3DSeriesDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QBar3DSeriesDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN26QBar3DSeriesDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBar3DSeriesDataVisForeign",
        "QML.Element",
        "QBar3DSeries",
        "QML.Foreign",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QBar3DSeries, use Bar3DSeries instea"
        "d."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
            {    6,    7 },
            {    8,    9 },
    });
    return QtMocHelpers::metaObjectData<QBar3DSeriesDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QBar3DSeriesDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QBar3DSeriesDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QBar3DSeriesDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QBar3DSeriesDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN30QScatter3DSeriesDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QScatter3DSeriesDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN30QScatter3DSeriesDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScatter3DSeriesDataVisForeign",
        "QML.Element",
        "QScatter3DSeries",
        "QML.Foreign",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QScatter3DSeries, use Scatter3DSerie"
        "s instead."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
            {    6,    7 },
            {    8,    9 },
    });
    return QtMocHelpers::metaObjectData<QScatter3DSeriesDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QScatter3DSeriesDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QScatter3DSeriesDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QScatter3DSeriesDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30QScatter3DSeriesDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN30QSurface3DSeriesDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QSurface3DSeriesDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN30QSurface3DSeriesDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSurface3DSeriesDataVisForeign",
        "QML.Element",
        "QSurface3DSeries",
        "QML.Foreign",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: QSurface3DSeries, use Surface3DSerie"
        "s instead."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
            {    6,    7 },
            {    8,    9 },
    });
    return QtMocHelpers::metaObjectData<QSurface3DSeriesDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QSurface3DSeriesDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QSurface3DSeriesDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QSurface3DSeriesDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30QSurface3DSeriesDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN23Q3DObjectDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto Q3DObjectDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN23Q3DObjectDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Q3DObjectDataVisForeign",
        "QML.Element",
        "Object3D",
        "QML.Foreign",
        "Q3DObject",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: Object3D."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<Q3DObjectDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Q3DObjectDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23Q3DObjectDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23Q3DObjectDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23Q3DObjectDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN29QAbstract3DAxisDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DAxisDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN29QAbstract3DAxisDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DAxisDataVisForeign",
        "QML.Element",
        "AbstractAxis3D",
        "QML.Foreign",
        "QAbstract3DAxis",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: AbstractAxis3D."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QAbstract3DAxisDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstract3DAxisDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QAbstract3DAxisDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QAbstract3DAxisDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QAbstract3DAxisDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN37QAbstract3DInputHandlerDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DInputHandlerDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN37QAbstract3DInputHandlerDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DInputHandlerDataVisForeign",
        "QML.Element",
        "AbstractInputHandler3D",
        "QML.Foreign",
        "QAbstract3DInputHandler",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: AbstractInputHandler3D."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QAbstract3DInputHandlerDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstract3DInputHandlerDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QAbstract3DInputHandlerDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QAbstract3DInputHandlerDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37QAbstract3DInputHandlerDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN31QAbstract3DSeriesDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstract3DSeriesDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN31QAbstract3DSeriesDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstract3DSeriesDataVisForeign",
        "QML.Element",
        "Abstract3DSeries",
        "QML.Foreign",
        "QAbstract3DSeries",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: Abstract3DSeries."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QAbstract3DSeriesDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstract3DSeriesDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QAbstract3DSeriesDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QAbstract3DSeriesDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QAbstract3DSeriesDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN32QAbstractDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN32QAbstractDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractDataProxyDataVisForeign",
        "QML.Element",
        "AbstractDataProxy",
        "QML.Foreign",
        "QAbstractDataProxy",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: AbstractDataProxy."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QAbstractDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstractDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QAbstractDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QAbstractDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32QAbstractDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN32QAbstractItemModelDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractItemModelDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN32QAbstractItemModelDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractItemModelDataVisForeign",
        "QML.Element",
        "AbstractItemModel",
        "QML.Foreign",
        "QAbstractItemModel",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: AbstractItemModel."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QAbstractItemModelDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstractItemModelDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QAbstractItemModelDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QAbstractItemModelDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32QAbstractItemModelDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN27QBarDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QBarDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN27QBarDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QBarDataProxyDataVisForeign",
        "QML.Element",
        "BarDataProxy",
        "QML.Foreign",
        "QBarDataProxy",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: BarDataProxy."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QBarDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QBarDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QBarDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QBarDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QBarDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN31QScatterDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QScatterDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN31QScatterDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScatterDataProxyDataVisForeign",
        "QML.Element",
        "ScatterDataProxy",
        "QML.Foreign",
        "QScatterDataProxy",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: ScatterDataProxy."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QScatterDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QScatterDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QScatterDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QScatterDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QScatterDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN31QSurfaceDataProxyDataVisForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QSurfaceDataProxyDataVisForeign::qt_create_metaobjectdata<qt_meta_tag_ZN31QSurfaceDataProxyDataVisForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSurfaceDataProxyDataVisForeign",
        "QML.Element",
        "SurfaceDataProxy",
        "QML.Foreign",
        "QSurfaceDataProxy",
        "QML.AddedInVersion",
        "256",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Trying to create uncreatable: SurfaceDataProxy."
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QSurfaceDataProxyDataVisForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QSurfaceDataProxyDataVisForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QSurfaceDataProxyDataVisForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QSurfaceDataProxyDataVisForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QSurfaceDataProxyDataVisForeignE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
